#This package should be included in all the scripts
use ForgeLogger;
use ForgeTM;

use Win32::OLE::Variant;


#---------------------------------------------------
# The following 3 environment variables should be 
# set in all the scripts to proper values
#---------------------------------------------------
$ENV{"LogFileName"} = "TicSmokeTest.html";
$ENV{"Title"} = "TIC Messaging Demo";
$ENV{"Description"} = "Smoke Test to check how TIC Tx/Rx Messages";

#---------------------------------------------------
# These functione should be called before using any 
# other functions in "ForgeLogger"
#---------------------------------------------------
ForgeLogger->Initialize();
my $TM = ForgeTM->new();

my @ExpectedMsg= ( 0x02, 0x01, 0xff, 0x01, 0x0, 0x00, 0x03, 0x00, 0x00, 0x01, 0x02, 0x03 );
my $ExpectedMsgSize= 12;
my @DataIn= (0x01, 0x02, 0x03);

my @RegNameArray= (
'EXCAUSE', 
'ILAT', 'IMASK', 'IPEND', 'lineEnd',
'P5', 'P4', 'P3', 'P2', 'lineEnd',
'P1', 'P0', 'lineEnd',
'R7', 'R6', 'R5', 'R4', 'lineEnd',
'R3', 'R2', 'R1', 'R0', 'lineEnd',
'I3', 'I2', 'I1', 'I0',  'lineEnd',
'M3', 'M2', 'M1', 'M0',  'lineEnd',
'B3', 'B2', 'B1', 'B0',  'lineEnd',
'L3', 'L2', 'L1', 'L0',  'lineEnd',
'LC0', 'LT0', 'LB0', 'lineEnd', 
'LC1', 'LT1', 'LB1', 'lineEnd', 
'FP', 'USP', 'lineEnd',
'ASTAT', 'SEQSTAT', 'SYSCFG', 'lineEnd',
'RETN', 'RETE', 'RETI', 'RETX', 'lineEnd'
);

#Enabling event callback feature
Win32::OLE->WithEvents($TM, \&CallbackOnReceiveTCSMessage  );
if( Win32::OLE->LastError() )
{
	print("Error:".Win32::OLE->LastError );
}

#Write the Test Id & Title to the logs
ForgeLogger->WriteHeading( "TIC", "TCS packet to TIC smoke test" );

# Send the TCS Message to target
$TM->SendTCSMessage( 2, 1, 255, \@DataIn);
if( Win32::OLE->LastError() )
{
   ForgeLogger->WriteMessage("Couldnt send message to TIC" );
   ForgeLogger->WriteMessage("Error:".Win32::OLE->LastError );
   ForgeLogger->WriteFail();
   print "Couldnt send TCS message to DSP\n";
}
else
{
    print "TCS message sent to DSP successfully\n";
	print "Waiting for TCS packet from Target (DSP)....\n";
	print "To stop press ctrl+c\n";

	#Waiting in a loop for response
	while(1)
	{
		Win32::OLE->SpinMessageLoop;
	}
	if( Win32::OLE->LastError() )
	{
	   print("Error:".Win32::OLE->LastError );
	   ForgeLogger->WriteMessage("Couldnt wait for message from target\n" );
	   ForgeLogger->WriteMessage("Error:".Win32::OLE->LastError );
	   ForgeLogger->WriteFail();
       print "TIC Smoke Test Failed\n";
	}
}


#Callback function for "OnReceiveTCSMessage" event
sub CallbackOnReceiveTCSMessage
{
	my ($tm,$Event,@Args) = @_;
	
	#Verify if the event that occured was 'OnReceiveTCSMessage'
	if( $Event eq 'OnReceiveTCSMessage' )
	{
		print "The Following TCS Packet was received:\n";
		
		#Get the msg that was sent from target
		my $ActualMsg =Variant(VT_VARIANT|VT_BYREF, $Args[0] );

		#Get the dimension of the array that contains the TCS msg 
		my $ArrayDim =  ($ActualMsg->Dim())[0];
		my $StartIndex = @$ArrayDim[0];
		my $EndIndex = @$ArrayDim[1];
		
		if( $ExpectedMsgSize != ($EndIndex-$StartIndex+1) )
		{
  		    ForgeLogger->WriteMessage("Received Msg size is different than expected" );
		    ForgeLogger->WriteFail();
            print "Received Msg size is different than expected\n";
            my $Mismatch = 1;
    	    my $temp32bitWord = 0;
			my $SIZE_TCS_HDR = 9;
			my $TIC_EXCEPTION_CODE = 0xFF;
			my $OFFSET_EXCAUSE = 13;
			my $OFFSET_REG_DATA = 17;

		   $temp32bitWord = $ActualMsg->Get($SIZE_TCS_HDR);
		   $temp32bitWord = $temp32bitWord | ($ActualMsg->Get($SIZE_TCS_HDR + 1) << 8);
		   $temp32bitWord = $temp32bitWord | ($ActualMsg->Get($SIZE_TCS_HDR + 2) << 16);
		   $temp32bitWord = $temp32bitWord | ($ActualMsg->Get($SIZE_TCS_HDR + 3) << 24);
		   printf "TIC_EXCEPTION_CODE = 0x%x\n", $temp32bitWord;


			#check if the TCS message is generated from EXception
			#if ($TIC_EXCEPTION_CODE == $ActualMsg->Get($SIZE_TCS_HDR))
			if ($TIC_EXCEPTION_CODE == $temp32bitWord)
			{
                print "DSP CORE GENERATED EXCEPTION and the cause of Eception is\n"; 
			    print "\n";

			    #print EXCAUSE
                my $regName = 0;
			    print $RegNameArray[$regName++];
			    printf " = 0x%x", $ActualMsg->Get($StartIndex + $OFFSET_EXCAUSE);
			    print "\n";
			    print "\n";

                print "DSP CORE Register Status is as below:\n";
			    print "\n";                                

                for( my $i=($StartIndex + $OFFSET_REG_DATA); $i<=$EndIndex;)
			    {
				   if ('lineEnd' eq $RegNameArray[$regName])
				   {
				     print "\n\n";
					 $regName++;
				   }
	     
				   print $RegNameArray[$regName++];
				   $temp32bitWord = $ActualMsg->Get($i++);
				   $temp32bitWord = $temp32bitWord | ($ActualMsg->Get($i++) << 8);
				   $temp32bitWord = $temp32bitWord | ($ActualMsg->Get($i++) << 16);
				   $temp32bitWord = $temp32bitWord | ($ActualMsg->Get($i++) << 24);
				   printf " = 0x%x   ", $temp32bitWord;
	   
			    }
                print "\n";
			}

   		   print "--------------------------------------------------\n";
		   if( $Mismatch == 1 )
		   {
		    	ForgeLogger->WriteMessage("Received Data mismatched the expected data" );
				ForgeLogger->WriteFail();
                print "Received Data mismatched the expected data\n"; 
                print "TIC Smoke Test Failed\n";
		   }

		}
		else
		{
			my $Mismatch = 0;
			for( my $i=$StartIndex; $i<=$EndIndex && $Mismatch == 0; $i++ )
			{
				print $ActualMsg->Get($i);
				
				#Do not check destination SWID cause it will be different
				#each time when the script is run
				if( $i != 5 )
				{
					if( $ActualMsg->Get($i) != $ExpectedMsg[$i] )
					{
						$Mismatch = 1;
					}
				}
				print "\n";
			}
			print "--------------------------------------------------\n";
			if( $Mismatch == 1 )
			{
				ForgeLogger->WriteMessage("Received Data mismatched the expected data" );
				ForgeLogger->WriteFail();
                print "Received Data mismatched the expected data\n"; 
                print "TIC Smoke Test Failed\n";
			}
			else
			{
				ForgeLogger->WriteMessage("Received Data matched the expected data" );
				ForgeLogger->WritePass();
                print "Received Data matched the expected data\n"; 
				print "TIC Smoke Test Passed\n";
			}
		}
	}
	exit;
}