/*  ********************************************************************
 *
 *  This file contains Analog Devices, Inc. proprietary information.
 *  Copyright  2002 by Analog Devices, Inc. All rights reserved.
 *
 *    File           : DIC.h
 *    Project        : ADSP-218x DSP Interrupt Controller driver   
 *    Author(s)      : Raheem Baig 
 *    Started        : 4th July 2004
 *    Description    : DSP DIC driver header file.
 *                     This file can be modified by DSP TAC developers 
 *                     only to change add more functions for DIC driver 
 *    
 *  Revision Control System (RCS) Information:
 *    $Workfile:   DIC.h  $
 *    $Revision:   1.1  $
 *     $Modtime:   Dec 26 2005 04:56:34  $
 *        $Date:   Jan 05 2006 08:56:34  $
 *      $Author:   mmanoj  $
 *     $Archive:   X:/Archive/Tools/Forge/Software/Target_Software/DSP/Library/TIC_DRV/218x/inc/DIC.h-arc  $
 *  
 *  ********************************************************************/

#ifndef DIC_H
#define DIC_H

#include "def218x.h"			/* ADSP218x constants & macros header file */

/*
 * constants and macros
 */
#define INT_GRP_8   0x8            /* Interrupt GROUP 8 */
#define INT_GRP_9   0x9            /* Interrupt GROUP 9 */
#define INT_GRP_10  0xA            /* Interrupt GROUP A */
#define INT_GRP_11  0xB            /* Interrupt GROUP B */
#define INT_GRP_12  0xC            /* Interrupt GROUP C */
#define INT_GRP_13  0xD            /* Interrupt GROUP D */
#define INT_GRP_14  0xE            /* Interrupt GROUP E */
#define INT_GRP_15  0xF            /* Interrupt GROUP F */

/* SWIs used in Forge TIC */
#define MCU_2_DSP_TIC_SWINUM  5    /* Software interrupt used for Forge TIC */
#define DSP_2_MCU_TIC_SWINUM  6	   /* Software interrupt used for Forge TIC */	

/*Number of Hardware Interrupt Groups */
#define DIC_NUM_OF_HW_GROUPS 0x8
/*
 * Variables and Externals
*/
/*
 ***************************************************************************
 *Function:     DIC_CfgDspIntCntl
 *Input:        None
 *Output:       None
 *Returns:      None
 *Regs Used:    AR, AY0, SR0, SR1, SE, I1, L1, M1
 *Description:  This subroutine is used to configure DSP interrupt controller to
 *              enable SWI'S used for TIC. This routine first Masks all the hard
 *              ware interrupts using hardware mask register and Mask all the
 *              software interrupts using SWI mask register Setup all the group
 *              priorities using Crossbar Configuration Register Finally set the
 *              lsb in the active interrupt register using RAU mode 10 and
 *              unmask IRQ2 interrupt to receive SWIs 
 ***************************************************************************
 */
#if (defined(VDSP_6_1))
.EXTERNAL DIC_CfgDSPIntCntl_; /* subroutine to configure DSP interrupt 
                               * controller */
#endif
#if (defined(VDSP_PP_3_5))
.EXTERN DIC_CfgDSPIntCntl_; /* subroutine to configure DSP interrupt 
                               * controller */
#endif

/*************************************************************************** 
 *Fuction:      DIC_IRQ2Isr
 *Input:        None
 *Output:       None
 *Returns:      None
 *Regs Used:    AR, AY0, SR1, SR0, I1, L1, M1, RWSM_DM_page00, RWSM_DM_page01
 *Description:  This subroutine is an ISR to IRQ2 interrupt This routine 
 *              indentifies the group number of the interrupt (i.e. 
 *              Hardware interrupt or software interrupt). Then it  finds 
 *              the priority of the interrupt and jump to the corresponding 
 *              interrupt service routine. Before returning for the ISR it
 *              clear the software interrupt and restores the registers.
*************************************************************************/

#if (defined(VDSP_6_1))
.EXTERNAL  DIC_IRQ2Isr_;		/* ISR to the DSP IRQ2 interrupt */
#endif

#if (defined(VDSP_PP_3_5))
.EXTERN  DIC_IRQ2Isr_;		/* ISR to the DSP IRQ2 interrupt */

#endif

/****************************************************************************

 *Function:       DIC_EnableHwi

 *Input:          HWI Group Number and Priority

 *Output:         None

 *Returns:        None

 *Regs Used:      SR0, SR1 and AR

 *Description:    This subroutine used to enable HWI whose group number and
 *                priority is passed to it via SR1 and SR0 registers, 
 *                respectively.

**************************************************************************/ 

#if (defined(VDSP_6_1))
.EXTERNAL  DIC_EnableHwi_;		/* Enables Hardware Interrupt */
#endif

#if (defined(VDSP_PP_3_5))
.EXTERN  DIC_EnableHwi_;		/* Enables Hardware Interrupt */

#endif

/****************************************************************************

 *Function:       DIC_EnableSwi

 *Input:          SWI Number

 *Output:         None

 *Returns:        None

 *Regs Used:      AR

 *Description:    This subroutine used to enable SWI whose number is passed
 *                to it via AR register.

****************************************************************************/

#if (defined(VDSP_6_1))
.EXTERNAL  DIC_EnableSwi_;      /* Enables Software Interrupt */
#endif

#if (defined(VDSP_PP_3_5))
.EXTERN  DIC_EnableSwi_;        /* Enables Software Interrupt */
#endif

/****************************************************************************

 *Function:       DIC_InstallInterruptHandler

 *Input:          Interrupt Handler Address, Priority of interrupt and Group

 *Output:         None

 *Returns:        None

 *Regs Used:      SR0, SR1, AR, AY0, AY1

 *Description:    This subroutine used to install an interrupt handler for 
 *                software and hardware interrupts. The 
 *                address of interrupt handler is passed via AR register.
 *                The priority and the group of the interrupt is passed via 
 *                SR0 and SR1 register, respectively. 
 **************************************************************************/ 

#if (defined(VDSP_6_1))
.EXTERNAL  DIC_InstallInterruptHandler_;    /* Install interrupt handler */
#endif

#if (defined(VDSP_PP_3_5))
.EXTERN  DIC_InstallInterruptHandler_;      /* Installs interrupt handler*/
#endif

/****************************************************************************

 *Function:       DIC_WriteCrossAssertReg

 *Input:          SWI Group Number and Priority

 *Output:         None

 *Returns:        None

 *Regs Used:      SR0, SR1, AR, AY1

 *Description:    This subroutine is used to write to the Cross Assert 
 *                 registers. The software interrupt group number and the 
 *                 priority is passed via SR0 and SR1 registers. 
****************************************************************************/

#if (defined(VDSP_6_1))
.EXTERNAL  DIC_GenXAssertSwi_;    /* Write to Cross Assert SWI Register */
#endif

#if (defined(VDSP_PP_3_5))
.EXTERN  DIC_GenXAssertSwi_;      /* Write to Cross Assert SWI Register */
#endif

/****************************************************************************

 *Function:       DIC_GenSwi

 *Input:          SWI Group Number and Priority

 *Output:         None

 *Returns:        None

 *Regs Used:      SR0, SR1, AR, AY1

 *Description:    This subroutine is used to write to the software interrupt
                  status register, to generate a software interrupt. The 
                  software interrupt group number and the priority is 
                  passed via SR0 and SR1 registers. 

****************************************************************************/
#if (defined(VDSP_6_1))
.EXTERNAL  DIC_GenSwi_;    /* Write to SWI Status Register */
#endif

#if (defined(VDSP_PP_3_5))
.EXTERN  DIC_GenSwi_;      /* Write to SWI Status Register */
#endif

#endif /* DIC_H /