/*
 *  HSLDrv.h
 */

#ifndef HSLDRV_H
#define HSLDRV_H

/*
 *  registers and bits of MSP430 HSL port
 */

#define     HSL_LAST_REG                    0x40
#define     HSL_DATA_REG                    0x41
#define     HSL_HEADER_REG                  0x42
#define     HSL_SINGLE_REG                  0x43
#define     HSL_CFG_REG                     0x44
#define     HSL_MASK00_REG                  0x45
#define     HSL_MASK01_REG                  0x46
#define     HSL_MASK10_REG                  0x47
#define     HSL_MASK11_REG                  0x48
#define     HSL_STACK_REG                   0x49

#define     HSL_CFG_PORT_EN                 0x0001
#define     HSL_CFG_TCU_EN                  0x0008
#define     HSL_CFG_DIV_4                   0x0004
#define     HSL_CFG_DIV_1                   0x0000
#define     HSL_CFG_LOGGING_EN              0x0020
#define     HSL_CFG_TRACE_EN                0x0010

/*
 *  primitive macros for writing to HSL registers in various ways
 */
 
#define HSL_WRITE_CFG(reg) \
    IO(HSL_CFG_REG) = reg
    
#define HSL_WRITE_MASK(mask,reg) \
    IO(HSL_MASK00_REG+mask) = reg
    
#define HSL_READ_MASK(mask,reg) \
    IO(HSL_MASK00_REG+mask) = reg
    
#define HSL_READ_CFG(reg) \
    reg = IO(HSL_CFG_REG)

#define HSL_GENERIC_WRITE_REG(mmio_loc, reg) \
    IO(mmio_loc) = reg
    
#define HSL_GENERIC_READ_REG(mmio_loc, reg) \
    reg = IO(mmio_loc)

#define LOG_SINGLE_REG(reg) \
    IO(HSL_SINGLE_REG) = reg
    
#define LOG_DATA_REG(reg) \
    IO(HSL_DATA_REG) = reg

#define LOG_SINGLE_DM(mem, trashreg) \
    trashreg = DM(mem); \
    LOG_DATA_REG(trashreg)
    
#define LOG_SINGLE_PM(mem, trashreg, ireg) \
    ireg = ^mem; \
    trashreg = PM(ireg, M7); \
    LOG_DATA_REG(reg)

#define LOG_VAL(val, trashreg) \
    trashreg = val; \
    LOG_DATA_REG(trashreg)

#define LOG_LAST_REG(reg) \
    IO(HSL_LAST_REG) = reg

#define LOG_HEADER_REG(reg) \
    IO(HSL_HEADER_REG) = reg

#define LOG_LAST_DM(mem, trashreg) \
    trashreg = DM(mem); \
    IO(HSL_LAST_REG) = trashreg

#endif