/*  ********************************************************************
 *
 *  This file contains Analog Devices, Inc. proprietary information.
 *  Copyright  2002 by Analog Devices, Inc. All rights reserved.
 *
 *    File           : LED.h
 *    Project        : ADSP-218x DSP LED driver   
 *    Author(s)      : Raheem Baig 
 *    Started        : 4th July 2004
 *    Description    : DSP LED drivers header file.
 *                     This file can be modified by DSP TAC developers 
 *                     only to change add more functions for LED driver 
 *    
 *  Revision Control System (RCS) Information:
 *    $Workfile:   LED.h  $
 *    $Revision:   1.1  $
 *     $Modtime:   Aug 19 2005 07:42:36  $
 *        $Date:   Jan 05 2006 08:56:36  $
 *      $Author:   mmanoj  $
 *     $Archive:   X:/Archive/Tools/Forge/Software/Target_Software/DSP/Library/TIC_DRV/218x/inc/LED.h-arc  $
 *  
 *  ********************************************************************/

#ifndef LED_H
#define LED_H

/*
 * includes
 */
#include "tic_drv.h"          /* TIC driver header file */
#include "def218x.h"			/* ADSP218x constants & macros header file */


/*
 * constants and macros
 */
#if (defined(ANVIL))
#define LW_LED_WORD_BASE_ADDR          0x0008
#define HW_LED_WORD_BASE_ADDR          0x1800
#define LW_LED_UPPER_HWORD_BASE_ADDR   0x000A
#define HW_LED_UPPER_HWORD_BASE_ADDR   0x1800
#define LW_LED_LOWER_HWORD_BASE_ADDR   0x0008
#define HW_LED_LOWER_HWORD_BASE_ADDR   0x1800
#define LED_LW_REG_ADDR                0x0C04    /* for 'RWSM_DM_page1' */
#define LED_HW_REG_ADDR                0x0C05    /* for 'RWSM_DM_page1' */
#endif

/*
 * Variables and Externals
*/
/*
 **************************************************************************
 *Function:     LED_Writeword
 *Input:        SR0 -- LW of 32bit word to display on LED
 *              SR1 -- LW of 32bit word to display on LED
 *Output:       32 bit Data is written onto LED's
 *Returns:      None
 *Regs Used:    SR0, SR1, RWSM_DM_page10, RWSM_DM_page11
 *Description:  This subroutine writes 32 bit data onto 32 LED's on ANVIL board
 *              This routine first saves the input parameters and registers used
 *              in this subroutine. It gets the address of LED wordbase to put
 *              in DM page registers using TicGetDspPageReg routine. The lower
 *              and higher words of input data are written to respective
 *              addresses of LED wordbase. Finally restore the registers used
 **************************************************************************
 */
#if (defined(VDSP_6_1))
.EXTERNAL LED_WriteWord_;  /* subroutine to display data on 32 LEDs */
#endif

#if (defined(VDSP_PP_3_5))
.EXTERN LED_WriteWord_;  /* subroutine to display data on 32 LEDs */
#endif

/*
 **************************************************************************
 *Function:      LED_WriteUpperHWord
 *Input:         AR -- 16 bit word to be displayed on LED
 *Output:        16 bit data is written on Upper LEDs on ANVIL
 *Returns:       None
 *Regs Used:     SR0, SR1, RWSM_DM_page10, RWSM_DM_page11
 *Description:   This subroutine is used to write a 16 bit data onto Upper LEDs
 *               on ANVIL board.This routine first saves the input parameters
 *               and registers used in this subroutine. It gets the address of
 *               LED wordbase to put in DM page registers using TicGetDspPageReg
 *               routine. The higher word is written to respective address of
 *               LED wordbase. Finally restores the registers used
 **************************************************************************
 */
#if (defined(VDSP_6_1))
.EXTERNAL LED_WriteUpperHWord_;  /* subroutine to display data on upper 16 LEDs */
#endif

#if (defined(VDSP_PP_3_5))
.EXTERN LED_WriteUpperHWord_;  /* subroutine to display data on upper 16 LEDs */
#endif

/*
 **************************************************************************
 *Function:      LED_WriteLowerHWord
 *Input:         AR -- 16 bit word to be displayed on LED
 *Output:        16 bit data is written on Lower LEDs on ANVIL
 *Returns:       None
 *Regs Used:     SR0, SR1, RWSM_DM_page10, RWSM_DM_page11
 *Description:   This subroutine is used to write a 16 bit data onto Lower LEDs
 *               on ANVIL board.This routine first saves the input parameters
 *               and registers used in this subroutine. It gets the address of
 *               LED wordbase to put in DM page registers using TicGetDspPageReg
 *               routine. The lower word is written to respective address of
 *               LED wordbase. Finally restores the registers used
 **************************************************************************
 */
#if (defined(VDSP_6_1))
.EXTERNAL LED_WriteLowerHWord_;  /* subroutine to display data on upper 16 LEDs */
#endif

#if (defined(VDSP_PP_3_5))
.EXTERN LED_WriteLowerHWord_;  /* subroutine to display data on upper 16 LEDs */
#endif

#endif /* LED_H /