/*
 *  LogChannel.h
 */

#ifndef LOGCHANNEL_H
#define LOGCHANNEL_H

#include "C:\dspdev\hsl\log API\sw - 218x\HSL TAC 1\Code\inc\LogChannel_ASCII.h"

/*
 *  define the I reg and M reg to be used in all Lc operations.  the idea is to
 *  avoid using I/M registers that are defined to be used for some system
 *  purpose (autobuffering, stack, etc).  note that the Lc API does not make any
 *  assumptions on an M register being equal to 1 at all times, it must be
 *  explicitly set in the code.
 */

#define LC_IREG0                            I1
#define LC_LREG0                            L1
#define LC_IREG1                            I2
#define LC_LREG1                            L2
#define LC_P_IREG0                          I5
#define LC_P_LREG0                          L5
#define LC_MREG0                            M2
#define LC_MREG1                            M3
#define LC_P_MREG0                          M5

#define LC_FALSE                            0
#define LC_TRUE                             1

/*
 *  maximum log channels definitions - this value is freely modifyable and can
 *  be increased/decreased as necessary.  note that LcCreate uses this value to
 *  determine if it can create a requested app channel.
 */

#define MAX_APP_LOGCHANNEL                  8
#define MAX_LOGCHANNEL                      (1+MAX_APP_LOGCHANNEL)

/*
 *  struct-centric offsets of the various log channels.  the first channel is
 *  the internal log channel, then there are a number of application log
 *  channels.  if the number of log channels must change, be sure to modify both
 *  the offsets defined here and MAX_APP_LOGCHANNEL.
 */
 
#define OFFSET_INT_LOGCHANNEL               0
#define OFFSET_APP_LOGCHANNEL_0             1
#define OFFSET_APP_LOGCHANNEL_1             2
#define OFFSET_APP_LOGCHANNEL_2             3
#define OFFSET_APP_LOGCHANNEL_3             4
#define OFFSET_APP_LOGCHANNEL_4             5
#define OFFSET_APP_LOGCHANNEL_5             6
#define OFFSET_APP_LOGCHANNEL_6             7
#define OFFSET_APP_LOGCHANNEL_7             8

/*
 *  LogChannel_t
 *
 *  log channel name, ID, option flags
 *
 *  note that the name is a packed char array.  the maximum length of a channel
 *  name is LENBYTES_LOGCHANNEL_NAME-1 characters long.  also note that the
 *  byte-sized equivalents are defined because the Lc API is byte-centric when
 *  transmitting values.
 */

#define LEN_LOGCHANNEL_NAME                 8
#define LEN_LOGCHANNEL_ID                   1
#define LEN_LOGCHANNEL_OPTS                 1
#define LEN_LOGCHANNEL                      (LEN_LOGCHANNEL_NAME + LEN_LOGCHANNEL_ID + LEN_LOGCHANNEL_OPTS)

#define LENBYTES_LOGCHANNEL_NAME            (LEN_LOGCHANNEL_NAME<<1)
#define LENBYTES_LOGCHANNEL_ID              (LEN_LOGCHANNEL_ID<<1)
#define LENBYTES_LOGCHANNEL_OPTS            (LEN_LOGCHANNEL_OPTS<<1)
#define LENBYTES_LOGCHANNEL                 (LEN_LOGCHANNEL<<1)

#define OFFSET_LOGCHANNEL_NAME              0
#define OFFSET_LOGCHANNEL_ID                (OFFSET_LOGCHANNEL_NAME+LEN_LOGCHANNEL_NAME)
#define OFFSET_LOGCHANNEL_OPTS              (OFFSET_LOGCHANNEL_ID+LEN_LOGCHANNEL_OPTS)

/*
 *  option flags for the OFFSET_LOGCHANNEL_OPTS parameter
 */

#define LC_FLAGS_STD_UNION_DECODE           0x0004
#define LC_FLAGS_TYPED_DATA                 0x0010  /* Channel name is data type. */
#define LC_FLAGS_EVENT                      0x0020  /* Channel name is enum name. */
                                                    /* header value 0..255 is "event enum name" */
#define LC_FLAGS_DISABLED                   0x8000
#define LC_FLAGS_HIGH_PRIO                  0x4000

/*
 *  offsets into g_pp_LogStructType, used for passing parameters to
 *  LcLogStructType
 */

#define LEN_LOGSTRUCT_PSZTEXT               8
#define LEN_LOGSTRUCT_PSZDATATYPENAME       8

#define OFFSET_LOGSTRUCT_PSZTEXT            0
#define OFFSET_LOGSTRUCT_PDATA              (OFFSET_LOGSTRUCT_PSZTEXT+LEN_LOGSTRUCT_PSZTEXT)
#define OFFSET_LOGSTRUCT_DWDATASIZE         (OFFSET_LOGSTRUCT_PDATA+1)
#define OFFSET_LOGSTRUCT_PSZDATATYPENAME    (OFFSET_LOGSTRUCT_DWDATASIZE+1)

#define LEN_LOGSTRUCT                       (OFFSET_LOGSTRUCT_PSZDATATYPENAME+LEN_LOGSTRUCT_PSZDATATYPENAME)

#define LOGSTRUCT_PDATA_BITPOS              15
#define LOGSTRUCT_PDATA_DM                  0x0000
#define LOGSTRUCT_PDATA_PM                  0x8000

/*
 *  since all passed parameters to LC API are handled through a single array,
 *  the length of this passed parameter array much be the length of the longest
 *  structure.
 */

#if (LEN_LOGSTRUCT > LEN_LOGCHANNEL)
#define LEN_LCDATA                          LEN_LOGSTRUCT
#else
#define LEN_LCDATA                          LEN_LOGCHANNEL
#endif


/*
 *  enumeration for internal log channel data IDs
 */

#define ENUM_LCID_CREATE                            0
#define ENUM_LCID_LOGSTRING                         1
#define ENUM_LCID_REPORTTSPARAM                     2       /* unused */
#define ENUM_LCID_LOGSTRUCTTYPE                     3
#define ENUM_LCID_REGISTERMODULEVERSION             4
#define ENUM_LCID_LOGSIMPLEEVENT                    5
#define ENUM_LCID_LOSTDATAMARKER                    6
#define ENUM_LCID_ATECCMD                           7

/*******************************************************************************
*  return codes for all LC API functions.  note that success is defined as
*  returning zero.  that way it is easy to check if the function call was a
*  success (AR = pass AR; if NE jump LcFail)
*******************************************************************************/

/*
 *  common return code for a nested log.  note that it is the only negative
 *  value return code, making it easy to check in the calling function if a
 *  nested log API call has occurred.
 */
 
#define RETCODE_NESTED_LOG                          -1

#define LC_RETURN \
    jump EndOfLcCall

/*
 *  return codes for LcInitialize
 */

#define RETCODE_LCINITIALIZE_SUCCESS                0
#define RETCODE_LCINITIALIZE_LC_WAS_INIT            1

/*
 *  return codes for LcCreate
 */
#define RETCODE_LCCREATE_SUCCESS                    0
#define RETCODE_LCCREATE_LC_NOT_INIT                1
#define RETCODE_LCCREATE_NO_CHAN_AVAILABLE          2

/*
 *  return codes for LcLogSimpleEvent
 */

#define RETCODE_LCLOGSIMPLEEVENT_SUCCESS            0
#define RETCODE_LCLOGSIMPLEEVENT_LC_NOT_INIT        1

/*
 *  return codes for LcLogEvent
 */

#define RETCODE_LCLOGEVENT_SUCCESS                  0
#define RETCODE_LCLOGEVENT_LC_NOT_INIT              1

/*
 *  return codes for LcLogStructType
 */

#define RETCODE_LCLOGSTRUCTTYPE_SUCCESS             0
#define RETCODE_LCLOGSTRUCTTYPE_LC_NOT_INIT         1

/*
 *  return codes for LcLogModuleVersion
 */

#define RETCODE_LCLOGMODULEVERSION_SUCCESS          0
#define RETCODE_LCLOGMODULEVERSION_LC_NOT_INIT      1

/*
 *  return codes for LcLogDataArray
 */

#define RETCODE_LCLOGDATAARRAY_SUCCESS              0
#define RETCODE_LCLOGDATAARRAY_LC_NOT_INIT          1

 
/*
 *  leftover stuff from MSP500 implementation.  kept in here for completeness
 *  and ease of port.  processor ID is transmitted back in log API header.  in
 *  this case the LC_ID_INTERNAL value is always 0.  on MSP500 where DSP and
 *  PC compatibility is desirable, these IDs have different values.
 */

#define THIS_PROCESSOR_PREFIX                       0

#define LC_ID_PID_POS                               11  /* 2 bits for proc */
#define LC_DATA_POS                                 13  /* 8 bits for data */
#define LC_SIZE_POS                                 21  /* 10 bits for size */
#define LC_SPC_POS                                  21  /* 11 bits for special buffers */

#define CHANNEL_ID_OFFSET                           (THIS_PROCESSOR_PREFIX << LC_ID_PID_POS)

/* Predefined channels numbers */
#define LC_FIXCH_INTERNAL                           OFFSET_INT_LOGCHANNEL
/*#define LC_FIRST_USER_CHAN                          OFFSET_APP_LOGCHANNEL_0*/

/* this value taken from MSP500 code.  not sure why it can't be 1. */
#define LC_FIRST_USER_CHAN                          32

/* Channel Ids for this processor */
#define LC_ID_INTERNAL                              (CHANNEL_ID_OFFSET + LC_FIXCH_INTERNAL)



#ifdef  M1_IS_1
#define COND_SET_M1_TO_1 !
#else
#define COND_SET_M1_TO_1 M1 = 1
#endif  /* M1_IS_1 */

/*******************************************************************************
*
*  macros for setting parameters passed to Lc functions.  to pass parameters to
*  the Lc API functions, a portion of memory is reserved and is automatically
*  read by the Lc function when it is called.  this reserved data array is
*  called g_pp_LcData (global passed parametered log channel data).  all
*  interaction with this memory should happen via the macros listed below.  also
*  note that these macros often require a register to be passed to them which
*  will be trashed (hence called the trashreg) - this can be any DREG except for
*  AF and MF.
*
*******************************************************************************/

/*
 *  LcLogSimpleEvent macros
 *
 *  LC_INIT_SIMPLE_EVENT_NAME
 *    max 4 chars - interpretted as ASCII by ATEC
 */
    
#define LC_INIT_SIMPLE_EVENT_NAME(char3, char2, char1, char0, trashreg) \
    trashreg = (char3<<8) | char2; \
    DM(g_pp_LcData+1) = trashreg; \
    trashreg = (char1<<8) | char0; \
    DM(g_pp_LcData+0) = trashreg

/*
 *  LcLogStructType macros
 *
 *  LC_INIT_LOGSTRUCTTYPE_PSZTEXT_1
 *  LC_INIT_LOGSTRUCTTYPE_PSZTEXT_2
 *  ...
 *  LC_INIT_LOGSTRUCTTYPE_PSZTEXT_15
 *  LC_INIT_LOGSTRUCTTYPE_PDATA
 *  LC_INIT_LOGSTRUCTTYPE_DWDATASIZE
 *  LC_INIT_LOGSTRUCTTYPE_PSZDATATYPENAME_1
 *  LC_INIT_LOGSTRUCTTYPE_PSZDATATYPENAME_2
 *  ...
 *  LC_INIT_LOGSTRUCTTYPE_PSZDATATYPENAME_15
 */

#define LC_INIT_LST_PSZTEXT_3(char2, char1, char0, trashreg) \
    call NullInitPszText; \
    trashreg = (char0); \
    DM(g_pp_LcData+OFFSET_LOGSTRUCT_PSZTEXT+1) = trashreg; \
    trashreg = (char2) | (char1<<8); \
    DM(g_pp_LcData+OFFSET_LOGSTRUCT_PSZTEXT+0) = trashreg
    
#define LC_INIT_LST_PSZTEXT_4(char3, char2, char1, char0, trashreg) \
    call NullInitPszText; \
    trashreg = (char3) | (char2<<8); \
    DM(g_pp_LcData+OFFSET_LOGSTRUCT_PSZTEXT+0) = trashreg; \
    trashreg = (char1) | (char0<<8); \
    DM(g_pp_LcData+OFFSET_LOGSTRUCT_PSZTEXT+1) = trashreg
    
#define LC_INIT_LST_PSZTEXT_5(char4, char3, char2, char1, char0, trashreg) \
    call NullInitPszText; \
    trashreg = (char4) | (char3<<8); \
    DM(g_pp_LcData+OFFSET_LOGSTRUCT_PSZTEXT+0) = trashreg; \
    trashreg = (char2) | (char1<<8); \
    DM(g_pp_LcData+OFFSET_LOGSTRUCT_PSZTEXT+1) = trashreg; \
    trashreg = (char0); \
    DM(g_pp_LcData+OFFSET_LOGSTRUCT_PSZTEXT+2) = trashreg
    
#define LC_INIT_LST_PSZTEXT_6(char5, char4, char3, char2, char1, char0, trashreg) \
    call NullInitPszText; \
    trashreg = (char5) | (char4<<8); \
    DM(g_pp_LcData+OFFSET_LOGSTRUCT_PSZTEXT+0) = trashreg; \
    trashreg = (char3) | (char2<<8); \
    DM(g_pp_LcData+OFFSET_LOGSTRUCT_PSZTEXT+1) = trashreg; \
    trashreg = (char1) | (char0<<8); \
    DM(g_pp_LcData+OFFSET_LOGSTRUCT_PSZTEXT+2) = trashreg

#define LC_INIT_LST_PDATA(memtype, addr) \
    AR = memtype; \
    AY0 = addr; \
    AR = AR or AY0; \
    DM(g_pp_LcData+OFFSET_LOGSTRUCT_PDATA) = AR

#define LC_INIT_LST_DWDATASIZE(size, trashreg) \
    trashreg = size; \
    DM(g_pp_LcData+OFFSET_LOGSTRUCT_DWDATASIZE) = trashreg
    
#define LC_INIT_LST_PSZDATATYPENAME_3(char2, char1, char0, trashreg) \
    call NullInitPszDataTypeName; \
    trashreg = (char0); \
    DM(g_pp_LcData+OFFSET_LOGSTRUCT_PSZDATATYPENAME+1) = trashreg; \
    trashreg = char2 | (char1<<8); \
    DM(g_pp_LcData+OFFSET_LOGSTRUCT_PSZDATATYPENAME+0) = trashreg
        
#define LC_INIT_LST_PSZDATATYPENAME_4(char3, char2, char1, char0, trashreg) \
    call NullInitPszDataTypeName; \
    trashreg = char3 | (char2<<8); \
    DM(g_pp_LcData+OFFSET_LOGSTRUCT_PSZDATATYPENAME+0) = trashreg; \
    trashreg = char1 | (char0<<8); \
    DM(g_pp_LcData+OFFSET_LOGSTRUCT_PSZDATATYPENAME+1) = trashreg
     
#define LC_INIT_LST_PSZDATATYPENAME_5(char4, char3, char2, char1, char0, trashreg) \
    call NullInitPszDataTypeName; \
    trashreg = (char0); \
    DM(g_pp_LcData+OFFSET_LOGSTRUCT_PSZDATATYPENAME+2) = trashreg; \
    trashreg = char2 | (char1<<8); \
    DM(g_pp_LcData+OFFSET_LOGSTRUCT_PSZDATATYPENAME+1) = trashreg; \
    trashreg = char4 | (char3<<8); \
    DM(g_pp_LcData+OFFSET_LOGSTRUCT_PSZDATATYPENAME+0) = trashreg
    
#define LC_INIT_LST_PSZDATATYPENAME_6(char5, char4, char3, char2, char1, char0, trashreg) \
    call NullInitPszDataTypeName; \
    trashreg = char1 | (char0<<8); \
    DM(g_pp_LcData+OFFSET_LOGSTRUCT_PSZDATATYPENAME+2) = trashreg; \
    trashreg = char3 | (char2<<8); \
    DM(g_pp_LcData+OFFSET_LOGSTRUCT_PSZDATATYPENAME+1) = trashreg; \
    trashreg = char5 | (char4<<8); \
    DM(g_pp_LcData+OFFSET_LOGSTRUCT_PSZDATATYPENAME+0) = trashreg
    
/*******************************************************************************
*
*  LcCreate macros
*
*  LC_INIT_PP_LOGCHANNEL_NAME_1
*  LC_INIT_PP_LOGCHANNEL_NAME_2
*  ...
*  LC_INIT_PP_LOGCHANNEL_NAME_15
*
*  LC_INIT_PP_LOGCHANNEL_OPTS
*
*******************************************************************************/
    
#define LC_INIT_PP_LOGCHANNEL_NAME_2(char1, char0, trashreg) \
    LC_IREG0 = ^g_pp_LcData; \
    call NullInitLogChannel; \
    trashreg = (char0<<8) | (char1); \
    DM(g_pp_LcData+0) = trashreg
     
#define LC_INIT_PP_LOGCHANNEL_NAME_4(char3, char2, char1, char0, trashreg) \
    LC_IREG0 = ^g_pp_LcData; \
    call NullInitLogChannel; \
    trashreg = (char0<<8) | (char1); \
    DM(g_pp_LcData+1) = trashreg; \
    trashreg = (char2<<8) | (char3); \
    DM(g_pp_LcData+0) = trashreg
         
#define LC_INIT_PP_LOGCHANNEL_NAME_5(char4, char3, char2, char1, char0, trashreg) \
    LC_IREG0 = ^g_pp_LcData; \
    call NullInitLogChannel; \
    trashreg = char0; \
    DM(g_pp_LcData+2) = trashreg; \
    trashreg = (char1<<8) | (char2); \
    DM(g_pp_LcData+1) = trashreg; \
    trashreg = (char3<<8) | (char4); \
    DM(g_pp_LcData+0) = trashreg
         
#define LC_INIT_PP_LOGCHANNEL_NAME_6(char5, char4, char3, char2, char1, char0, trashreg) \
    LC_IREG0 = ^g_pp_LcData; \
    call NullInitLogChannel; \
    trashreg = (char0<<8) | char1; \
    DM(g_pp_LcData+2) = trashreg; \
    trashreg = (char2<<8) | (char3; \
    DM(g_pp_LcData+1) = trashreg; \
    trashreg = (char4<<8) | (char5; \
    DM(g_pp_LcData+0) = trashreg

#define LC_INIT_PP_LOGCHANNEL_OPTS(flags, trashreg) \
    trashreg = flags; \
    DM(g_pp_LcData+OFFSET_LOGCHANNEL_OPTS) = trashreg

/*
 *  LC_JUMP_ON_FAIL
 *    place after call to Lc function.  if they return a non-zero value (zero
 *    means success for all Lc functions), jump to passed failure function.
 */

#define LC_JUMP_ON_FAIL(func) \
    AR = pass AR; \
    if NE jump func

#endif  /* LOGCHANNEL_H */
