/*  ********************************************************************
 *
 *  This file contains Analog Devices, Inc. proprietary information.
 *  Copyright  2002 by Analog Devices, Inc. All rights reserved.
 *
 *    File           : SDO.h
 *    Project        : ADSP-218x DSP Seven Segment driver   
 *    Author(s)      : Raheem Baig 
 *    Started        : 4th July 2004
 *    Description    : DSP SDO drivers header file.
 *                     This file can be modified by DSP TAC developers 
 *                     only to change add more functions for SDO driver 
 *    
 *  Revision Control System (RCS) Information:
 *    $Workfile:   SDO.h  $
 *    $Revision:   1.1  $
 *     $Modtime:   Aug 19 2005 07:43:28  $
 *        $Date:   Jan 05 2006 08:56:38  $
 *      $Author:   mmanoj  $
 *     $Archive:   X:/Archive/Tools/Forge/Software/Target_Software/DSP/Library/TIC_DRV/218x/inc/SDO.h-arc  $
 *  
 *  ********************************************************************/

#ifndef SDO_H
#define SDO_H

/*
 * includes
 */
#include "tic_drv.h"          /* TIC driver header file */
#include "def218x.h"            /* ADSP218x constants & macros header file */


/*
 * constants and macros
 */
#if (defined(CLASSIC))
#define LW_SDO_BASE_ADDR  0x0020
#define HW_SDO_BASE_ADDR  0x1800
#define SDO_REG_ADDR      0x0C10  /* for 'RWSM_DM_page1' */
#elif (defined(ANVIL))
#define LW_SDO_BASE_ADDR  0x0014
#define HW_SDO_BASE_ADDR  0x1800
#define SDO_REG_ADDR      0x0C0A  /* for 'RWSM_DM_page1' */
#endif
/*
 * Variables and Externals
*/
/*
 **************************************************************************
 *Function:      SDO_WriteValue
 *Input:         AR - Data to be displayed on 7 segments
 *Output:        None
 *Returns:       None
 *Regs Used:     SR0, SR1, RWSM_DM_page10, RWSM_DM_page11
 *Description:   This subroutine is used to write the input data onto 7 segment
 *               display. This routine first saves the input data and registers
 *               used in this routine It gets the addresses of 7 segment
 *               display to put in DM page registers by using the subroutine
 *               TicGetDspPageReg. Using these addresses data will be written
 *               onto 7 segment display. Finally restores the registers used.
 **************************************************************************
 */
#if (defined(VDSP_6_1))
.EXTERNAL SDO_WriteValue_;  /* subroutine to display data byte on 7 seg display */
#endif

#if (defined(VDSP_PP_3_5))
.EXTERN SDO_WriteValue_;  /* subroutine to display data byte on 7 seg display */
#endif

#endif /* SDO_H /