/*****************************************************************************     
 *    This file contains Analog Devices, Inc. proprietary information.
 *    Copyright 2000 by Analog Devices, Inc.  All rights reserved.
 * 
 *    File        : tcs.h
 *    Project     : FORGE
 *    Author(s)   : Denise DiBiase
 *    Started     : 10-08-2001
 *    Description : Header file for FORGE Target Communication System.
 * 
 *    Revision Control System (RCS) Information:
 *    $Workfile:   tcs.h  $
 *    $Revision:   1.1  $
 *     $Modtime:   Dec 01 2005 01:30:46  $   
 *        $Date:   Jan 12 2006 01:26:22  $   
 *      $Author:   Ashirwad  $   
 *     $Archive:   X:/Archive/Tools/Forge/Software/Target_Software/MCU/Target/Code/inc/tcs.h-arc  $
 *   
 ****************************************************************************/
#ifndef TCS_H
   #define TCS_H
            
 
/*****************************************************************************
 * System Include Files
 ****************************************************************************/


/*****************************************************************************
 * Local Include Files
 ****************************************************************************/
#include "ftypes.h"
#include "tvcpext.h"

/*****************************************************************************
 * Global Declarations
 ****************************************************************************/

#define  TCS_PACKET_POOL_SIZE    96   /* number of TCS Packet Pool slots */
#define  TCS_MSG_MAX_SIZE        255   /* maximum TCS Message body size */
#define  TRWE_MSG_HDR_SIZE       7    /* Size of the TRWE Msg Header */
#define  TRWE_MSG_MAX_SIZE      (TCS_MSG_MAX_SIZE - TRWE_MSG_HDR_SIZE)

#define SIZE_VCP_HDR    (sizeof(VCP_HDR))
#define SIZE_VCP_TRLR   (sizeof(VCP_TRLR))


/*****************************************************************************
 * 
 *           Name:   TX_MSG_TYPE
 *           Type:   enum
 * Scope Of Usage:   Intialized in TvcpInit(), set in TvcpTransmitMsg(), 
 *					 checking in TcsTxPacketIsr1()
 *
 *    Description:   Specifies the type of message transmitted last.
 *
 ****************************************************************************/
typedef  enum tagTX_MSG_TYPE
{
      NO_MSG,    	/* no message, initial condition */
      VCP_MSG,
      VCP_RESP,      
      TCS_MSG
} TX_MSG_TYPE;

/*****************************************************************************
 * 
 *           Name:   TCS_ID
 *           Type:   Struct
 * Scope Of Usage:   May be used by all software modules that use TCS Messages.
 *    Description:   TCS IDs for the Device, Processor and Software module 
 *                   fields of the TCS Header.
 *
 ****************************************************************************/
typedef  struct tagTCS_ID
{
      UINT8    dev;        /* Device ID */
      UINT8    proc;       /* Processor ID */
      UINT8    sw;         /* Software Module ID */ 
} TCS_ID;

#define SIZE_TCS_ID		(sizeof(struct tagTCS_ID))

/*****************************************************************************
 * 
 *           Name:   TCS_MSG_HDR
 *           Type:   Struct
 * Scope Of Usage:   Used by all software modules that use TCS Messages.
 *    Description:   TCS IDs for the Device, Processor and Software module
 *                   fields of the TCS Header.  
 *
 ****************************************************************************/
typedef  struct tagTCS_MSG_HDR
{
   TCS_ID   src;           /* Source */                 
   TCS_ID   dst;           /* Destination */
   UINT8    length;        /* TCS Message Body Length */
   UINT8    reserved1;     /* Reserved for future use */ 
   UINT8    reserved2;     /* Reserved for future use */
} TCS_MSG_HDR;  

#define SIZE_TCS_MSG_HDR      (sizeof(TCS_MSG_HDR))

/*****************************************************************************
 * 
 *           Name:   TCS_PACKET
 *           Type:   Struct
 * Scope Of Usage:   Used by all software modules that use TCS Messages.
 *    Description:   TCS IDs for the Device, Processor and Software module 
 *                   fields of the TCS Header.  
 *
 ****************************************************************************/
typedef struct tagTCS_PACKET
{ 
   VCP_HDR        vcpHeader;
   TCS_MSG_HDR    tcsMsgHdr;       
   UCHAR          tcsMessage[TCS_MSG_MAX_SIZE];
   VCP_TRLR       vcpTrailer; 
} TCS_PACKET;
 
#define SIZE_TCS_PACKET       (sizeof(TCS_PACKET))

/*****************************************************************************
 * 
 *           Name:   TCS_MESSAGE
 *           Type:   Struct
 * Scope Of Usage:   Used when only the TCS Message is needed.
 *    Description:   TCS Message HEader and Message only.
 *
 ****************************************************************************/
typedef struct tagTCS_MESSAGE
{ 
   TCS_MSG_HDR    tcsMsgHdr;       
   UCHAR          tcsMessage[TCS_MSG_MAX_SIZE];
} TCS_MESSAGE;

#define SIZE_TCS_MESSAGE		(sizeof(struct tagTCS_MESSAGE))

/*****************************************************************************
 * 
 *           Name:   TCS_DEVICE_IDS
 *           Type:   Struct
 * Scope Of Usage:   Used when only device IDs are needed.
 *    Description:   Source & destination IDs for Device field of TCS Header.  
 *                   
 ****************************************************************************/
typedef  struct tagTCS_DEVICE_IDS
{
   UINT8    src;           /* Source */                 
   UINT8    dst;           /* Destination */
} TCS_DEVICE_IDS;  

#define SIZE_TCS_DEVICE_IDS		(sizeof(tagTCS_DEVICE_IDS))
 
/*****************************************************************************
 *
 *        Macro:  TCS_COPY_UINT32
 *
 *   Parameters:  sOURCE - Byte addressed Source pointer
 *                dEST - Byte addressed Destination pointer
 *
 * Side Effects:  none
 *
 *  Description:  Copy byte by byte from source ptr to destination pointer 4 
 *                times. Needed for removing WORD or HALFWORD variables from TCS
 *                messages into local variables
 *
 ****************************************************************************/
#define TCS_COPY_UINT32(sOURCE,dEST)  \
{  ((UINT8*) (dEST))[0] = ((UINT8*) (sOURCE))[0];  \
   ((UINT8*) (dEST))[1] = ((UINT8*) (sOURCE))[1];  \
   ((UINT8*) (dEST))[2] = ((UINT8*) (sOURCE))[2];  \
   ((UINT8*) (dEST))[3] = ((UINT8*) (sOURCE))[3];}

/*****************************************************************************
 *
 *        Macro:  TCS_COPY_UINT16
 *
 *   Parameters:  sOURCE - Byte addressed Source pointer
 *                dEST - Byte addressed Destination pointer
 *
 * Side Effects:  none
 *
 *  Description:  Copy byte by byte from source ptr to destination pointer 2 
 *                times. Needed for removing WORD or HALFWORD variables from TCS
 *                messages into local variables
 *
 ****************************************************************************/
#define TCS_COPY_UINT16(sOURCE,dEST)  \
{  ((UINT8*) (dEST))[0] = ((UINT8*) (sOURCE))[0];  \
   ((UINT8*) (dEST))[1] = ((UINT8*) (sOURCE))[1];}

/*****************************************************************************
 *
 *        Macro:  Global macro name
 *
 *   Parameters:  parameter 1 - description   
 *                parameter 2 - description
 *
 * Side Effects:  Describe any side effects here or NONE
 *
 *  Description:  Description
 *  
 ****************************************************************************/

#endif /* TCS_H */
