/*  ********************************************************************
 *
 *  This file contains Analog Devices, Inc. proprietary information.
 *  Copyright  2002 by Analog Devices, Inc. All rights reserved.
 *
 *    File           : tic_drv.h
 *    Project        : ADSP-218x DSP TIC driver   
 *    Author(s)      : Raheem Baig 
 *    Started        : Dec 9th 2002
 *    Description    : DSP TIC drivers header file.
 *                     This file can be modified by DSP TAC developers 
 *                     only to change either 
 *                     1. Modifiers(M-regs) or Index registers (I-regs)
 *                       2. Stack size
 *                       3. Stack address (if the stack address location has
 *                          conflict with DSP TAC addresses. 
 *    
 *  Revision Control System (RCS) Information:
 *    $Workfile:   tic_drv.h  $
 *    $Revision:   1.1  $
 *     $Modtime:   Dec 20 2005 04:02:54  $
 *        $Date:   Jan 05 2006 08:56:46  $
 *      $Author:   mmanoj  $
 *     $Archive:   X:/Archive/Tools/Forge/Software/Target_Software/DSP/Library/TIC_DRV/218x/inc/tic_drv.h-arc  $
 *  
 *  ********************************************************************/

#ifndef TIC_DRV_H
#define TIC_DRV_H

#include "def218x.h"  /* ADSP218x constants & macros header file */
#include "DIC.h"
#include "SDO.h"

/* constants and macros */
/* modifiers used in DSP TIC driver code */
#define M_1                  M7
#define M_n1                 M6
#define M_user               M3
                                
/* Index register used in DSP TIC driver code */
#define I_SP                 I6

/* Length register used in DSP TIC driver code */
#define L_SP                 L6

#define TRUE                  1
#define FALSE                 0

#define DSP_STACK_SIZE        128  /* optimal stack size used in Forge 
                                    * DSP TIC drivers 
                                    */
#define ARM_PROCESSOR         0   /* TIC number (ID) of ARM processor */
#define DSP_PROCESSOR         1      /* TIC number (ID) of DSP processor */    
#define NUM_PROCS             2   /* number of processors in Forge Target */
#define NUM_TICS              (NUM_PROCS - 1)  /* number of TIC modules */
#define NUM_STS_FLAS          2     /* number of half words (16 bit), 
                                       One word is for status flag to write TCS
                                       message into the shared memory by MCU 
                                       and another is to write by DSP  */
#define TCS_MSGBODY_MAX_SIZE  255   /* maximum TCS Message body size */
#define TCS_HDR_SIZE          9     /* maximum TCS Header size */
#define SIZE_TCS_MESSAGE      (TCS_MSGBODY_MAX_SIZE + TCS_HDR_SIZE)
#define TIC_SM_SIZE           2 * (SIZE_TCS_MESSAGE)  /*number of half words */
#define NUM_IDs               3  /* Device ID, Processor ID and SW module ID */
#define OFFSET_TCS_MSG_LEN    6  /* 6 halfwords */
#define OFFSET_TCS_MSG_BODY   9  /* 9 halfwords */


#if (defined(RUN_FORGE_IN_SRAM))
#define LW_TIC_ADDR_VECT_PTR  0x0020  /* lower word of pointer to  Share memory
                                         address vector */
#define HW_TIC_ADDR_VECT_PTR  0x4000  /* higher word of pointer to  Share memory
                                         address vector */
#else
#if (defined(RUN_FORGE_IN_FLASH))
#define LW_TIC_ADDR_VECT_PTR  0x0020  /* lower word of pointer to  Share memory
                                         address vector */
#define HW_TIC_ADDR_VECT_PTR  0x0100  /* higher word of pointer to  Share memory
                                         address vector */
#else            
#define LW_TIC_ADDR_VECT_PTR  0x0020  /* lower word of pointer to  Share memory
                                         address vector */
#define HW_TIC_ADDR_VECT_PTR  0x0000  /* higher word of pointer to  Share memory
                                         address vector */
#endif
#endif

#define NUM_ADDR_STORED    2  /* num of 32bit addresses stored */
#define NUM_HWORDS_32BIT   2  /* 
                               * num of HalfWords (16bit) in 32bit address 
                               * since ADI218x DSP is 16bit processor 
                               */
/*
 * Variables and Externals
*/
#if (defined(VDSP_6_1))
.EXTERNAL stackTopAddr;             /* dataword to store stack top address */
.EXTERNAL  DSPStack;             /* stack used for Forge DSP TIC drivers */

.EXTERNAL newTcsMsgFlag;         /* flag to show a new TCS message is Received*/
.EXTERNAL tcsHeader;             /* TCS Header buffer */
.EXTERNAL ticRxTcsMsg;             /* Received TCS message buffer */
.EXTERNAL ticTxTcsMsg;             /* Transmitted TCS message buffer */
.EXTERNAL tcsMsgSize;
.EXTERNAL addr_tcsMsg;             /* pointer to the TCS message buffer */
#endif

#if (defined(VDSP_PP_3_5))
.EXTERN stackTopAddr;             /* dataword to store stack top address */
.EXTERN  DSPStack;             /* stack used for Forge DSP TIC drivers */

.EXTERN newTcsMsgFlag;         /* flag to show a new TCS message is Received*/
.EXTERN tcsHeader;             /* TCS Header buffer */
.EXTERN ticRxTcsMsg;             /* Received TCS message buffer */
.EXTERN ticTxTcsMsg;             /* Transmitted TCS message buffer */
.EXTERN tcsMsgSize;
.EXTERN addr_tcsMsg;             /* pointer to the TCS message buffer */
#endif

/* 
 * Global function definitions 
 */
/*************************************************************************
 *Function:       main
 *Input:          None
 *Output:         None
 *Returns:        None
 *Regs Used:      None
 *Description:    This is the main function in which all the functions are 
 *                written and this is called when the DSP is reset.It returns 
 *                nothing 
**************************************************************************/
#if (defined(VDSP_6_1))
.EXTERNAL main_;                   /* ISR to DSP reset */
#endif

#if (defined(VDSP_PP_3_5))
.EXTERN main_;                   /* ISR to DSP reset */
#endif
/************************************************************************* 
 Function:     TicDSPMCUIsr_
 Input:        None
 Output:       None
.Returns:      None
.Regs Used:    AR
.Description: This Subroutine is an ISR when ever SWI is generated by MCU
        to DSP This Reads the TCS Message from MCU using TicRdMCU2DSPTcsMsg
        subroutine and sets NewTcsMsgFlag 
*************************************************************************/
#if (defined(VDSP_6_1))
.EXTERNAL TicDSPMCUIsr_;           /* driver to copy TCS message received 
                                    * from MCU 
                                    */
#endif

#if (defined(VDSP_PP_3_5))
.EXTERN TicDSPMCUIsr_;           /* driver to copy TCS message received 
                                    * from MCU 
                                    */
#endif

/************************************************************************* 
 *Function:     TIcSendTcsMsg2MCU
 *Input:        None
 *Output:       None
 *Regs Used:    AR
 *Description: This subroutine will send TCS message to MCU This function check
 *             whether DSP to MCU status flag is set or not if it is set wait
 *             until it is again cleared once it resets it sends the TCS Message
 *             using TicWrDSP2MCUTcsMsg function before which packing of TCS
 *             message header is done after sending the message it will again
 *             set the DSP to MCU status flag and calls TicGenXassertSwi to 
 *             generate a SWI
*************************************************************************/
#if (defined(VDSP_6_1))
.EXTERNAL TicSendTcsMsg2MCU_;      /* driver to send TCS message to MCU */
#endif

#if (defined(VDSP_PP_3_5))
.EXTERN TicSendTcsMsg2MCU_;      /* driver to send TCS message to MCU */
#endif

/************************************************************************* 
 *Function:    TicUnPackTcsHdr
 *Input:       AR - addr_tcsMsg (address of either ticRxTcsMsg or ticTxTcsMsg)
 *Output:      TCS message header
 *Returns:     None
 *Regs Used:   AR, I0, L0, I1, L1, M1
 *Description: This Subroutine extracts the TCS MSG Header from TCS message 
 *             Packet TCS header Extracted from the input TCS message is copied 
 *             to TCS Header buffer.
*************************************************************************/
#if (defined(VDSP_6_1))
.EXTERNAL TicUnPackTcsHdr_;            /* driver to unpack TCS message header */
#endif

#if (defined(VDSP_PP_3_5))
.EXTERN TicUnPackTcsHdr_;            /* driver to unpack TCS message header */
#endif

/************************************************************************* 
 *Function:    TicPackTcsHdr
 *Input:       AR - addr_tcsMsg (address of either ticRxTcsMsg or ticTxTcsMsg)
 *Output:      None
 *Returns:     None
 *Regs Used:   AR, I0, L0, I1, L1, M1
 *Description: This subroutine is used to Pack TCS MSG Header into the TCS
 *             message buffer which is in Internal DSP memory This routine puts 
 *             the Tcs header from header Buffer to TCS message buffer.

*************************************************************************/
#if (defined(VDSP_6_1))
.EXTERNAL TicPackTcsHdr_;            /* driver to pack TCS message header */
#endif
#if (defined(VDSP_PP_3_5))
.EXTERN TicPackTcsHdr_;            /* driver to pack TCS message header */
#endif

/************************************************************************** 
 *Function:     TicSwapIds
 *Input:        None
 *Output:       None
 *Returns:      None
 *Regs Used:    AR, AY0, I0, L0, I1, L1, M1, M_user
 *Description:  After the unpacking the header from TCS message, this subroutine
 *             is used to Swap Source and Destination ID's in the Header Read
 *             source device, source processor, source SW module, Destination
 *             device, destination processor and destination SW mosule ID's and
 *             swap the respective source and destination id's
*************************************************************************/
#if (defined(VDSP_6_1))
.EXTERNAL TicSwapIds_;                /* driver to SWAP source & destination IDs
                                     *  of TCS message 
                                     */
#endif

#if (defined(VDSP_PP_3_5))
.EXTERN TicSwapIds_;                /* driver to SWAP source & destination IDs
                                     *  of TCS message 
                                     */
#endif

/************************************************************************* 
 *Function:      TicGetDSP2MCUStsFlag
 *Input:         None
 *Output:        Status Flags in AR
 *Returns:       None
 *Regs Used:     AR, SR0, SR1, I0, L0, M1, M_user, RWSM_DM_page10, RWSM_DM_page11
 *Description:   This subroutine is used to get the status flags for DSP2MCU
 *               messaging This subroutine first gets the address for DSP2MCU
 *               status flags. From these addresses get the values to put in DM
 *               page registers by using TicGetDSPPageReg subroutine and using
 *              TicSetDSPDmAddr subroutine return those values into AR.
*************************************************************************/
#if (defined(VDSP_6_1))
.EXTERNAL TicGetDSP2MCUStsFlag_;   /* driver to read DSP2MCU messaging flag */
#endif

#if (defined(VDSP_PP_3_5))
.EXTERN TicGetDSP2MCUStsFlag_;   /* driver to read DSP2MCU messaging flag */
#endif

/*************************************************************************** 
 *Function:     TicGetMCU2DSPStsFlag
 *Input:        None
 *Output:       Status Flags in AR 
 *Returns:      None
 *Regs Used:    AR, SR0, SR1, I0, L0, M1, M_user, RWSM_DM_page10, RWSM_DM_page11
 *Description:  This subroutine is used to get the status flags for MCU2DSP 
 *              messaging This subroutine first gets the address for MCU2DSP 
 *              status flags From these addresses get the values to put in DM 
 *              page registers by using TicGetDSPPageReg subroutine and using 
 *              TicSetDSPDmAddr subroutine return those values into AR.

*************************************************************************/
#if (defined(VDSP_6_1))
.EXTERNAL TicGetMCU2DSPStsFlag_;   /* driver to read MCU2DSP messaging flag */
#endif

#if (defined(VDSP_PP_3_5))
.EXTERN TicGetMCU2DSPStsFlag_;   /* driver to read MCU2DSP messaging flag */
#endif

/*************************************************************************
 *Function:      TicSetDSP2MCUStsFlag
 *Input:         None
 *Output:        None
 *Returns:       None
 *Regs Used:     AR, SR0, SR1, I0, L0, M1, M_user
 *Description:   This subroutine is used to Set DSP2MCU tcs msg Status Flag This
 *               subroutine first gets the addresses of DSP2MCU tcs msg Status
 *               Flags Set the status flags by using TicSetStsFlag Subroutine.
*************************************************************************/
#if (defined(VDSP_6_1))
.EXTERNAL TicSetDSP2MCUStsFlag_;   /* driver to set DSP2MCU messaging flag */
#endif

#if (defined(VDSP_PP_3_5))
.EXTERN TicSetDSP2MCUStsFlag_;   /* driver to set DSP2MCU messaging flag */
#endif

/*************************************************************************
 *Function:     TicClrMCU2DSPStsFlag
 *Input:        None
 *Output:       None
 *Returns:      None
 *Regs Used:    AR, SR0, SR1, I0, L0, M1, M_user
 *Description:  This Subroutine is used to clear MCU2DSP tcs msg STATUS FLAG
 *              This subroutine first gets the addresses of MCU2DSP tcs msg
 *              Status Flags Clear the status flags by using TicClrStsFlag
 *              Subroutine.
*************************************************************************/
#if (defined(VDSP_6_1))
.EXTERNAL TicClrMCU2DSPStsFlag_;   /* Subroutine to CLEAR MCU2DSP tcs msg 
                                    * STATUS FLAG 
                                    */
#endif

#if (defined(VDSP_PP_3_5))
.EXTERN TicClrMCU2DSPStsFlag_;   /* Subroutine to CLEAR MCU2DSP tcs msg 
                                    * STATUS FLAG 
                                    */
#endif

/*************************************************************************
 *Function:     TicSetStsFlag
 *Input:        SR0 - lw_stsFlagAddr
 *              SR1 - hw_stsFlagAddr
 *Output:       TCS messaging Status Flag has set 
 *Returns:      None
 *Regs Used:    AR, SR0, SR1, I0, L0, M1, RWSM_DM_page10, RWSM_DM_page11
 *Description:  This subroutine is used to set the Status Flags Here the input 
 *              address pointers are passed and get the values to put in DM page
 *              registrs by using TicGetDSPPageReg subroutine These values are 
 *              given as inputs to TicsetDmAddr routine to get the result is 
 *              stored in AR register.Using this AR Register the flags are set
*************************************************************************/
#if (defined(VDSP_6_1))
.EXTERNAL TicSetStsFlag_;           /* driver to set any status flag */
#endif

#if (defined(VDSP_PP_3_5))
.EXTERN TicSetStsFlag_;           /* driver to set any status flag */
#endif

/**************************************************************************
 *Function:     TicClrStsFlag 
 *Input:        SR0 - lw_stsFlagAddr
 *              SR1 - hw_stsFlagAddr
 *Output:       TCS messaging Status Flag has set
 *Returns:      None
 *Regs Used:    AR, SR0, SR1, I0, L0, M1, RWSM_DM_page10, RWSM_DM_page11
 *Description:  This subroutine is used to clear the Status Flags here the input
 *              address pointers are passed and get the values to put in DM page
 *              registrs by using TicGetDSPPageReg subroutine These values are 
 *              given as inputs to TicsetDmAddr routine to  get the result is 
 *              stored in AR register.Using this AR Register the flags are 
 *              cleared 
*************************************************************************/
#if (defined(VDSP_6_1))
.EXTERNAL TicClrStsFlag_;           /* driver to clear any status flag */    
#endif

#if (defined(VDSP_PP_3_5))
.EXTERN TicClrStsFlag_;           /* driver to clear any status flag */    
#endif

/***************************************************************************
 *Function:      TicGenXassertSwi
 *Input:         TicSwiNum
 *Output:        None
 *Returns:       None
 *Regs Used:     AR, I0, SR0, SR1, SE 
 *Description:   This subroutine will generate Cross Assert SWI This interrupt
 *               generation can be done by setting the SWI bit in cross assert 
 *               register
*************************************************************************/
#if (defined(VDSP_6_1))
.EXTERNAL TicGenXassertSwi_;       /* driver to generate a cross asserted SWI */
#endif

#if (defined(VDSP_PP_3_5))
.EXTERN TicGenXassertSwi_;       /* driver to generate a cross asserted SWI */
#endif

/*************************************************************************
 *Function:        TicGetDSPPageReg
 *Input:           SR0 - lw_DSPbus32bit_addr  ;address LS word
 *                 SR1 - hw_DSPbus32bit_addr  ;address MS word
 *Output:          None
 *Returns:         LS word of DM/PM page registers
 *                 MS word of DM/PM page registers
 *Regs Used:       SR0, SR1
 *Description:     This subroutine will get Page register values to put in DM 
 *                 page Registers.Get the Lower word by masking the Upper byte 
 *                 in HW of 32 bit address in hercules and all athena versions 
 *                 and for all Hermes versions Mask all 11 bits in the Lower 
 *                 word of page Register 
 **************************************************************************/
#if (defined(VDSP_6_1))
.EXTERNAL TicGetDSPPageReg_;       /* driver to set a single DM page register */
#endif

#if (defined(VDSP_PP_3_5))
.EXTERN TicGetDSPPageReg_;       /* driver to set a single DM page register */
#endif

/***************************************************************************
 *Function:     TicSetDmPageRegs
 *Input:        SR0 - lw_DSPbus32bit_addr  ;address LS word
 *              SR1 - hw_DSPbus32bit_addr  ;address MS word
 *Output:       None
 *Returns:      Updated address of LS word to SR0
 *              Updated address of MS word to SR1
 *Regs Used:    AR, I1, L1, M1 
 *Description:  This subroutine is used to initialize [set] all the six DM page
 *              Registers All DM page Registers are Set to New Value Get the 
 *              first page address and put in a loop to set all the pages with 
 *              new addresses.
*************************************************************************/

#if (defined(VDSP_6_1))
.EXTERNAL TicSetDmPageRegs_;       /* driver to set all DM page registers */
#endif

#if (defined(VDSP_PP_3_5))
.EXTERN TicSetDmPageRegs_;       /* driver to set all DM page registers */
#endif

/***************************************************************************
 *Function:     TicSetDSPDMAddr
 *Input:        SR1 - Page register ID
 *              SR0 - DSP_32bit_BUS address LS word
 *Output:       None
 *Returns:      14 Bit DSP Bus address
 *Regs Used:    AR, SR0, SR1
 *Description:  This subroutine is used to convert the DSP_32bit_bus address LS
 *              word to 14 bit DSP Bus address by masking the Upper 5 bits and
 *              lsb in the lower word of DSPBUS Return both updated page ID and 
 *              14 bit DSPBUS Address 
*************************************************************************/
#if (defined(VDSP_6_1))
.EXTERNAL TicSetDSPDmAddr_;           /* driver to set 14bit DSP DM address */ 
#endif

#if (defined(VDSP_PP_3_5))
.EXTERN TicSetDSPDmAddr_;           /* driver to set 14bit DSP DM address */ 
#endif

/*************************************************************************
 *Function:     TicSetDSPPMAddr
 *Input:        SR1 - Page register ID
 *              SR0 - DSP_32bit_BUS address LS word
 *Output:       None
 *Returns:      14 Bit DSP Bus address
 *Regs Used:    AR, SR0, SR1
 *Description:  This subroutine is used to convert the DSP_32bit_bus address LS
 *              word to 14 bit DSP Bus address by masking the Upper 5 bits and
 *              lsb in the lower word of DSPBUS For PM set Bit 13 in SR0
 *              Since ExtRAM addresses are different for PM Return both updated
 *              page ID and 14 bit DSPBUS Address

*************************************************************************/
#if (defined(VDSP_6_1))
.EXTERNAL TicSetDSPPmAddr_;           /* driver to set 14bit DSP PM address */     
#endif

#if (defined(VDSP_PP_3_5))
.EXTERN TicSetDSPPmAddr_;           /* driver to set 14bit DSP PM address */     
#endif

/***************************************************************************
 *Function:     TicPushRegsOntoStack
 *Input:        None
 *Output:       None
 *Returns:      None
 *Regs Used:    AR, AY0, SR0, SR1, I0, I1, L0, L1, M1, RWSM_DM_page10, 
 *              RWSM_DM_page11,RWSM_DM_page20, RWSM_DM_page21.
 *Description:  This subroutine is used to push the Regs used in that subroutine
 *              to stack for saving the data in that registers L_sp is the 
 *              Length of stack I_sp is the stack pointer which is modified by 
 *              M_n1 to push [write] the Registers onto stack in Data Memory
*************************************************************************/
#if (defined(VDSP_6_1))
.EXTERNAL TicPushRegsOntoStack_;   /* driver to save (PUSH) registers used onto
                                    * the stack 
                                    */
#endif

#if (defined(VDSP_PP_3_5))
.EXTERN TicPushRegsOntoStack_;   /* driver to save (PUSH) registers used onto
                                    * the stack 
                                    */
#endif

/****************************************************************************
 *Function:      TicPopRegsOntoStack 
 *Input:         None
 *Output:        None
 *Returns:       None
 *Regs Used:     AR, AY0, SR0, SR1, I0, I1, L0, L1, M1, RWSM_DM_page10,
 *               RWSM_DM_page11,RWSM_DM_page20, RWSM_DM_page21.
 *Description:   This subroutine is used to pop the Regs used in that subroutine
 *               from stack for retrieving the data in that registers I_sp is 
 *               the stack pointer which is modified by M_1 to Pop [Read] the 
 *               Registers From stack in Data Memory
*************************************************************************/
#if (defined(VDSP_6_1))
.EXTERNAL TicPopRegsFromStack_;    /* driver to restore (POP) registers used 
                                    * from the stack 
                                    */
#endif

#if (defined(VDSP_PP_3_5))
.EXTERN TicPopRegsFromStack_;    /* driver to restore (POP) registers used 
                                    * from the stack 
                                    */
#endif

/*************************************************************************** 
 *Function:     TicSetLedTDO
 *Input:        None
 *Output:       None
 *Returns:      None
 *Regs_used:    AR
 *Description:  This Tic driver subroutine is used to set TDO LED (DSP_PF[0]) on
 *              DBB This can be done by setting the 1st bit of PF DATA.
*************************************************************************/
#if (defined(VDSP_6_1))
.EXTERNAL TicSetLedTDO_;           /* subroutine to set TDO LED (DSP_PF[0]) on 
                                    * DBB 
                                    */
#endif

#if (defined(VDSP_PP_3_5))
.EXTERN TicSetLedTDO_;           /* subroutine to set TDO LED (DSP_PF[0]) on 
                                    * DBB 
                                    */
#endif

/***************************************************************************
 *Function:      TicClrLedTDO
 *Input:         None
 *Output:        None
 *Returns:       None
 *Regs_used:     AR 
 *Description:   This Tic driver subroutine is used to Clear TDO LED (DSP_PF[0])
 *               on DBB This can be done by clearing the 1st bit of PF DATA.
 ***************************************************************************/
#if (defined(VDSP_6_1))
.EXTERNAL TicClrLedTDO_;           /* subroutine to clear TDO LED (DSP_PF[0]) 
                                    * on DBB 
                                    */
#endif

#if (defined(VDSP_PP_3_5))
.EXTERN TicClrLedTDO_;           /* subroutine to clear TDO LED (DSP_PF[0]) 
                                    * on DBB 
                                    */
#endif

/***************************************************************************
 *Function:     TicTglLedTDO
 *Input:        None
 *Output:       None
 *Returns:      None
 *Regs_used:    AR
 *Description:  This Tic driver subroutine is used to Toggle TDO LED (DSP_PF[0])
 *              on DBB This can be done by Toggling the 1st bit of PF DATA.
 ***************************************************************************/
#if (defined(VDSP_6_1))
.EXTERNAL TicTglLedTDO_;           /* subroutine to toggle TDO LED (DSP_PF[0]) 
                                    * on DBB 
                                    */
#endif

#if (defined(VDSP_PP_3_5))
.EXTERN TicTglLedTDO_;           /* subroutine to toggle TDO LED (DSP_PF[0]) 
                                    * on DBB 
                                    */
#endif

/***************************************************************************
 *Function:     TicSetLedTDI
 *Input:        None
 *Output:       None
 *Returns:      None
 *Regs_used:    AR
 *Description:  This Tic driver subroutine is used to set TDO LED (DSP_PF[1]) on
 *              DBB This can be done by setting the 2nd bit of PF DATA.
 ***************************************************************************/
#if (defined(VDSP_6_1))
.EXTERNAL TicSetLedTDI_;            /* subroutine to set TDI LED (DSP_PF[1]) 
                                    * on DBB 
                                    */
#endif
#if (defined(VDSP_PP_3_5))
.EXTERN TicSetLedTDI_;            /* subroutine to set TDI LED (DSP_PF[1]) 
                                    * on DBB 
                                    */
#endif

/***************************************************************************
 *Function:      TicClrLedTDI
 *Input:         None
 *Output:        None
 *Returns:       None
 *Regs_used:     AR 
 *Description:   This Tic driver subroutine is used to Clear TDO LED (DSP_PF[1])
 *               on DBB This can be done by clearing the 2nd bit of PF DATA.
 ***************************************************************************/
#if (defined(VDSP_6_1))
.EXTERNAL TicClrLedTDI_;            /* subroutine to clear TDI LED (DSP_PF[1]) 
                                    * on DBB 
                                    */
#endif
#if (defined(VDSP_PP_3_5))
.EXTERN TicClrLedTDI_;            /* subroutine to clear TDI LED (DSP_PF[1]) 
                                    * on DBB 
                                    */
#endif

/***************************************************************************
 *Function:     TicTglLedTDI
 *Input:        None
 *Output:       None
 *Returns:      None
 *Regs_used:    AR
 *Description:  This Tic driver subroutine is used to Toggle TDO LED (DSP_PF[1])
 *              on DBB This can be done by Toggling the 2nd bit of PF DATA.
 ***************************************************************************/
#if (defined(VDSP_6_1))
.EXTERNAL TicTglLedTDI_;           /* subroutine to toggle TDI LED (DSP_PF[1]) 
                                    * on DBB 
                                    */
#endif
#if (defined(VDSP_PP_3_5))
.EXTERN TicTglLedTDI_;           /* subroutine to toggle TDI LED (DSP_PF[1]) 
                                    * on DBB 
                                    */
#endif

#if (defined(AD6527_8_9) || defined(AD6758))

/***************************************************************************
 *Function:       TicSetLedTMS
 *Input:          None
 *Output:         None
 *Returns:        None
 *Regs Used:      AR
 *Description:    This driver subroutine will set the TMS LED (DSP_PF[2]) on DBB
 *                This can be done by setting the 3rd bit of PFDATA 
 ***************************************************************************/
#if (defined(VDSP_6_1))
.EXTERNAL TicSetLedTMS_;    /* subroutine to set TMS LED (DSP_PF[2]) on DBB */
#endif

#if (defined(VDSP_PP_3_5))
.EXTERN TicSetLedTMS_;    /* subroutine to set TMS LED (DSP_PF[2]) on DBB */
#endif

/***************************************************************************
 *Function:b      TicClrLedTMS
 *Input:          None
 *Output:         None
 *Returns:        None
 *Regs Used:      AR
 *Description:    This driver subroutine will clear the TMS LED (DSP_PF[2]) on 
 *                DBB This can be done by clearing the 3rd bit of PFDATA
 ***************************************************************************/
#if (defined(VDSP_6_1))
.EXTERNAL TicClrLedTMS_;    /* subroutine to clear TMS LED (DSP_PF[2]) on DBB */
#endif

#if (defined(VDSP_PP_3_5))
.EXTERN TicClrLedTMS_;    /* subroutine to clear TMS LED (DSP_PF[2]) on DBB */
#endif

/***************************************************************************
 *Function:b      TicTglLedTMS
 *Input:          None
 *Output:         None
 *Returns:        None
 *Regs Used:      AR
 *Description:    This driver subroutine will Toggle the TMS LED (DSP_PF[2]) on 
 *                DBB This can be done by Toggling the 3rd bit of PFDATA
 ***************************************************************************/
#if (defined(VDSP_6_1))
.EXTERNAL TicTglLedTMS_;   /* subroutine to toggle TMS LED (DSP_PF[2]) on DBB */
#endif

#if (defined(VDSP_PP_3_5))
.EXTERN TicTglLedTMS_;   /* subroutine to toggle TMS LED (DSP_PF[2]) on DBB */
#endif

#endif /*#if (defined(AD6527_8_9) || defined(AD6758))*/

#endif /* TIC_DRV_H /



