/*
 *  CSRstack.dsp
 *
 *  context save/restore code for handling stacks
 */

.MODULE                               mod_CSRstack;

#define LOOP_STACK_EMPTY                    0x0040
#define COUNT_STACK_EMPTY                   0x0004
#define PC_STACK_EMPTY                      0x0001

.GLOBAL dicSaveStacks, dicRecallStacks;
.GLOBAL ProgforPop_h, ProgforPop_l, ProgforPush_h, ProgforPush_l;
.EXTERNAL dlddicEndRecallStacks, dlddicEndSaveStacks;


dicSaveStacks:
dicSaveCntrStack:
  ar = 0;                                         /* SC always: cycles: 6                                           */
  ay0 = COUNT_STACK_EMPTY;
saveCntrStackStart:
  ax0 = sstat;
  NONE = ax0 AND ay0;
  IF NE JUMP saveCntrStackExit;
  ar = ar + 1;                                    /* keep a count of the number of loops we save                    */
  ax0 = cntr;                                     /* SC per_CNTR: cycles: 5                                         */
  POP cntr;
  DM (i4, m7)= ax0;
  JUMP saveCntrStackStart;
saveCntrStackExit:
  DM (i4, m7)= ar;                                /* save the number of saved loops                                 */




  ar = 0;                                         /* SC always: cycles: 6                                           */
  ay0 = LOOP_STACK_EMPTY;
saveLoopStackStart:
  ax0 = sstat;                                    /* keep on reading the status until the...                        */
  NONE = ax0 AND ay0;                             /* LOOP_STACK_EMPTY bit becomes true                              */
  IF NE JUMP saveLoopStackExit;
  ar = ar + 1;                                    /* keep a count of the number of loops we save                    */
  ax0 = toploopstackl;                            /* SC per_TOPLOOPSTACK: cycles: 6                                 */
  ax1 = toploopstackh;
  DM (i4, m7)= ax0;
  DM (i4, m7)= ax1;
  JUMP saveLoopStackStart;
saveLoopStackExit:
  DM (i4, m7)= ar;                                /* rememder the number of saved loops                             */

  !JUMP dlddicEndSaveStacks;                       /* SC always: cycles: 1                                           */

/*
 *  extra code to save LIStack (klio-core only).  LISSTAT resides at DM(0x3fe9)
 *  and is read (and the contents of LIS saved) until LISSTAT = 0x0001
 *  indicating LIS is empty.  if this code is run on a klio-core DSP,
 *  opcodes are written into the labels ProgforPop_h and ProgforPop_l in
 *  mldinit.dsp.  if this code is run on a non klio-core DSP, the opcode at
 *  label saveLIStackStart is written in mldinit to be "AX0 = 1".  this forces
 *  the test to be true and the LIS save loop is exitted immediately.
 */

#if 1
#ifndef DISABLE_DIONE_CORE_COMPATIBILITY    
    AR = 0;
    AY0 = 0x0001;                           /* LIStack empty status bit */
saveLIStackStart:
    AX0 = DM(0x3fe9);                       /* in hermes, modified to AX0=1 */
    NONE = AX0 and AY0;
    if NE jump saveLIStackExit;
    jump saveLIStackExit;
    AR = AR + 1;

    /* opcode + pop for high/low portions of LIS */
    ProgforPop_l:       nop;
    nop;
    DM(I4, M7) = AX0;
    
    ProgforPop_h:       nop;
    nop;
    DM(I4, M7) = AX0;

    /* jum to reread and find out if LIS is empty yet */
    jump saveLIStackStart;

saveLIStackExit:
    DM(I4, M7) = AR;                        /* store # LIS popped */
    
    !DM(numLISpush) = AR;
    
#endif
#endif

  ar = 0;                                         /* SC always: cycles: 6                                           */
  ay0 = PC_STACK_EMPTY;
savePCStackStart:
  ax0 = sstat;
  NONE = ax0 AND ay0;
  IF NE JUMP savePCStackExit;
  ar = ar + 1;                                    /* keep a count of the number of PCs we save                      */
  ax0 = toppcstack;                               /* SC save_PC: cycles: 5                                          */
  DM (i4, m7)= ax0;
#   ifdef COS_LOG_STACK_PUSH
#     ifdef ENABLE_TROY_STACK_IN_DM0
#       error Cannot write to the logger when in DM0 context
#     endif /* ENABLE_TROY_STACK_IN_DM0 */
  ay1 = LG_PP_STACK_PUSH;                         /* SC hsl: cycles: 16                                             */
  ax1 = i4;
  CALL LgStdMes2;                                 /* logs AX0,AX1                                                   */
#   endif 
  JUMP savePCStackStart;
savePCStackExit:
  DM (i4, m7)= ar;                                /* rememder the number of saved PCs                               */


  JUMP dlddicEndSaveStacks;                       /* SC always: cycles: 1                                           */

/* - - */

/**********************************************************************************************************************
 * 
 * =FUNCTION dicRecallStacks
 * =DESC     This subroutine restores all the values associated with a HW loop.
 * =INPUT: I4
 * Uses: AR, AX0.
 * =OUTPUT: I4
 * Assumes: M4 = -1, L4 = 0;
 *********************************************************************************************************************/


dicRecallStacks:
  ay1 = sstat;

  ar = DM (i4, m4);                               /* how many PCs were saved ?                                      */
  ar = PASS ar;                                   /* if no CNTR loops were saved, go to LOOP stacks                 */
  IF EQ JUMP recallPCStackExit;
recallPCStackStart:
  ax0 = DM (i4, m4);
  toppcstack = ax0;
  ar = ar - 1;
  IF NE JUMP recallPCStackStart;


recallPCStackExit:

/*
 *  extra code to restore LIStack (klio-core only).  for an MSP430 part that
 *  does not have a klio-core, AR will always be 0, forcing an immediate exit
 *  from this popping loop.
 */
 
#if 1
#ifndef DISABLE_DIONE_CORE_COMPATIBILITY
    AR = DM(I4, M4);
    
    AR = pass AR;
    if EQ jump recallLIStackExit;

recallLIStackStart:
    dis INTS;

    AX0 = DM(I4, M4);
    nop;
    ProgforPush_h:  nop;                /* location where opcode is written */

    AX0 = DM(I4, M4);
    nop;
    ProgforPush_l:  nop;                /* location where opcode is written */
    ena INTS;
     
    AR = AR - 1;
    if NE jump recallLIStackStart;
#endif
#endif
recallLIStackExit:

/*
 *  end of LIS code
 */

!dicRecallStacks:

  ar = DM (i4, m4);                               /* how many loops were saved ?                                    */
  ar = PASS ar;                                   /* if no loops were saved,                                        */
  IF EQ JUMP recallLoopStackExit;                 /* go home                                                        */
recallLoopStackStart:
  ax1 = DM (i4, m4);
  ax0 = DM (i4, m4);
  DIS INTS;
  toploopstackh = ax1;
  toploopstackl = ax0;
  ENA INTS;
  ar = ar - 1;
  IF NE JUMP recallLoopStackStart;
recallLoopStackExit:



  ar = DM (i4, m4);                               /* how many CNTR loops were saved ?                               */
  ar = PASS ar;                                   /* if no CNTR loops were saved, go to LOOP stacks                 */
  IF EQ JUMP recallCntrStackExit;
recallCntrStackStart:
  ax0 = DM (i4, m4);
  cntr = ax0;
  ar = ar - 1;
  IF NE JUMP recallCntrStackStart;
recallCntrStackExit:



  JUMP dlddicEndRecallStacks;












#if 0
dicSaveStacks:
dicSaveCntrStack:
   AR = 0;                                      !SC always: cycles: 6
   AY0 = COUNT_STACK_EMPTY;
saveCntrStackStart:
   AX0 = SSTAT;
   NONE = AX0 AND AY0;
   IF NE JUMP saveCntrStackExit;
   AR = AR + 1;                        /* keep a count of the number of loops we save */
   AX0 = CNTR;                                  !SC per_CNTR: cycles: 5
   POP CNTR;
   DM(I4,M7) = AX0;
   JUMP saveCntrStackStart;
saveCntrStackExit:
   DM(I4,M7) = AR;                     /* save the number of saved loops */

    

   AR = 0;                                      !SC always: cycles: 6
   AY0 = LOOP_STACK_EMPTY;
saveLoopStackStart:
   AX0 = SSTAT;                        /* keep on reading the status until the...*/
   NONE = AX0 AND AY0;                 /* LOOP_STACK_EMPTY bit becomes true */
   IF NE JUMP saveLoopStackExit;
   AR = AR + 1;                        /* keep a count of the number of loops we save */
   AX0 = TOPLOOPSTACKL;                         !SC per_TOPLOOPSTACK: cycles: 6
   AX1 = TOPLOOPSTACKH;
   DM(I4,M7) = AX0;
   DM(I4,M7) = AX1;
   
    ProgforPop_h:       nop;
    nop;
    DM(I4, M7) = AX0;
    
    ProgforPop_l:       nop;
    nop;
    DM(I4, M7) = AX0;

   
   JUMP saveLoopStackStart;
saveLoopStackExit:
   DM(I4,M7) = AR;                     /* rememder the number of saved loops */

   AR = 0;                                      !SC always: cycles: 6
   AY0 = PC_STACK_EMPTY;
savePCStackStart:
   AX0 = SSTAT;
   NONE = AX0 AND AY0;
   IF NE JUMP savePCStackExit;
   AR = AR + 1;              /* keep a count of the number of PCs we save */
   AX0 = TOPPCSTACK;                            !SC save_PC: cycles: 5
   DM(I4,M7) = AX0;
#ifdef COS_LOG_STACK_PUSH
# ifdef ENABLE_TROY_STACK_IN_DM0
#  error Cannot write to the logger when in DM0 context
# endif /* ENABLE_TROY_STACK_IN_DM0 */
   AY1 = LG_PP_STACK_PUSH;                      !SC hsl: cycles: 16
   AX1 = I4;
   CALL LgStdMes2;          /* logs AX0,AX1 */
#endif
   JUMP savePCStackStart;
savePCStackExit:
   DM(I4,M7) = AR;                     /* rememder the number of saved PCs */


   JUMP dlddicEndSaveStacks;                    !SC always: cycles: 1

/*- -*/

/*+********************************************************************
 *
 * =FUNCTION dicRecallStacks
 * =DESC     This subroutine restores all the values associated with a HW loop.
 * =INPUT: I4
 * Uses: AR, AX0.
 * =OUTPUT: I4
 * Assumes: M4 = -1, L4 = 0;
 *******************************************************************+*/


dicRecallStacks:
   AY1 = SSTAT;

   AR = DM(I4,M4);                     /* how many PCs were saved ? */
   AR = PASS AR;                       /* if no CNTR loops were saved, go to LOOP stacks */
   IF EQ JUMP recallPCStackExit;
recallPCStackStart:
   AX0 = DM(I4,M4);
   TOPPCSTACK = AX0;
   AR = AR - 1;
   IF NE JUMP recallPCStackStart;


recallPCStackExit:
   AR = DM(I4,M4);                     /* how many loops were saved ? */
   AR = PASS AR;                       /* if no loops were saved, */
   IF EQ JUMP recallLoopStackExit;     /* go home */
recallLoopStackStart:
   AX1 = DM(I4,M4);
   AX0 = DM(I4,M4);
   DIS INTS;
     TOPLOOPSTACKH = AX1;
     TOPLOOPSTACKL = AX0;
     
     AX0 = DM(I4, M4);
     nop;
     ProgforPush_l:  nop;

     AX0 = DM(I4, M4);
     nop;
     ProgforPush_h:  nop;
     
     
     
   ENA INTS;
   AR = AR - 1;
   IF NE JUMP recallLoopStackStart;
recallLoopStackExit:

   AR = DM(I4,M4);                     /* how many CNTR loops were saved ? */
   AR = PASS AR;                       /* if no CNTR loops were saved, go to LOOP stacks */
   IF EQ JUMP recallCntrStackExit;
recallCntrStackStart:
   AX0 = DM(I4,M4);
   CNTR = AX0;
   AR = AR - 1;
   IF NE JUMP recallCntrStackStart;
recallCntrStackExit:



   JUMP dlddicEndRecallStacks;
#endif

!.entry ProgforPop_h, ProgforPop_l, ProgforPush_h, ProgforPush_l;
!ProgforPop_h:
!ProgforPop_l:
!ProgforPush_h:
!ProgforPush_l:
    nop;

.ENDMOD;
