/*
 *  LogChannel.dsp
 */

.MODULE                                 m_LogChannel;

#include    "LogChannel.h"
#include    "LogChannelVars.h"
#include    "HSLDrv.h"

#include    "NopUtil.h"

.ENTRY      LcInitialize;

.ENTRY      LcLogStructType;
.ENTRY      LcLogModuleVersion;
.ENTRY      LcLogSimpleEvent;

.ENTRY      LcCreate;
.ENTRY      LcLogEvent;
.ENTRY      LcLogDataArray;
.ENTRY      LcLogMultiData;

.ENTRY      SendHSLATECInitSeq;

/*
 *  LcInitialize
 *
 *  - check if already called.  if true, exit
 *  - init internal log channel
 *  - write to global indicating internal channel exists
 *
 *  registers destroyed
 *
 *  AR, ASTAT, I1, M1
 */

LcInitialize:
    AR = DM(gboLcInitialized);
    AR = pass AR;
    if EQ jump StartLcInitialize;
    AR = RETCODE_LCINITIALIZE_LC_WAS_INIT;
    rts;
    

StartLcInitialize:
    AR = LC_TRUE;
    DM(gboLcInitialized) = AR;
   
    I1 = ^g_LogChannel;
    COND_SET_M1_TO_1;
    call NullInitLogChannel;

    AR = LC_TRUE;
    DM(gboLcLogChannelCreate) = AR;
    AR = RETCODE_LCINITIALIZE_SUCCESS;
    rts;

LcLogStructType:
    rts;
LcLogModuleVersion:
    rts;

/*
 *  LcLogSimpleEvent
 *
 *  log a single 32 bit word
 */

LcLogSimpleEvent:
    AR = DM(gboLcInitialized);
    AR = pass AR;
    if NE jump LcLogSimpleEvent_LcInitOK;
    AR = RETCODE_LCLOGSIMPLEEVENT_LC_NOT_INIT;
    rts;

/*
 *  important!  the log API is 32 bit centric.  however, in this case we can
 *  cheat a bit because we know that (ENUM_LCID_LOGSIMPLEEVENT << LC_DATA_POS)
 *  will not extend beyond 16 bits.  therefore the first word written is 0, and
 *  the second word is this header word
 */

LcLogSimpleEvent_LcInitOK:

    AY0 = 0;
    !AY0 = 1;
    LOG_DATA_REG(AY0);
    AY0 = ENUM_LCID_LOGSIMPLEEVENT << LC_DATA_POS;
    LOG_DATA_REG(AY0);
    
    LOG_SINGLE_DM(g_pp_dwEventName, AY0);
    LOG_SINGLE_DM(g_pp_dwEventName+1, AY0);
    
    AR = RETCODE_LCLOGSIMPLEEVENT_SUCCESS;
    rts;

/*
 *  LcCreate
 *
 *  create a new application log channel.  parameters are passed in the
 *  following global arrays (and therefore must be initialized before the call
 *  to this function.
 *
 *  g_pp_LogChannel
 *
 *  modified values
 *
 *    AR, AY0, ASTAT, MY0, MR0, MR1, MR2, MSTAT, M2, I1, I2
 *
 *  return values
 *
 */

LcCreate:
    AR = DM(gboLcInitialized);
    AR = pass AR;
    if NE jump LcCreate_LcInitOK;
    AR = RETCODE_LCCREATE_LC_NOT_INIT;
    rts;
    
LcCreate_LcInitOK:
    AR = DM(g_NextChannelIdToUse);
    AY0 = MAX_LOGCHANNEL;
    NONE = AR - AY0;
    if GT jump LcCreate_LcChanAvailable;
    AR = RETCODE_LCCREATE_NO_CHAN_AVAILABLE;
    rts;

LcCreate_LcChanAvailable:
    AR = DM(g_NextChannelIdToUse);
    MY0 = LEN_LOGCHANNEL_T;
    I1 = mstat;
    dis m_mode;
    MR = AR * MY0 (UU);
    mstat = I1;
    AY0 = ^g_LogChannel;
    AR = MR1 + AY0;
    I1 = AR;
    I2 = ^g_pp_LogChannel;
    COND_SET_M1_TO_1;
    CNTR = LEN_LOGCHANNEL_T;
    do LCC1 until CE;
      AR = DM(I2, M1);
LCC1: DM(I1, M1) = AR;
    
    AR = DM(g_NextChannelIdToUse);
    AR = AR + 1;
    DM(g_NextChannelIdToUse) = AR;
    
    /* log channel creation */

    AR = RETCODE_LCCREATE_SUCCESS;
    rts;
    
   

LcLogEvent:
    AR = DM(gboLcInitialized);
    AR = pass AR;
    if NE jump LcLogEvent_LcInitOK;
    AR = RETCODE_LCLOGEVENT_LC_NOT_INIT;
    rts;
    
LcLogEvent_LcInitOK:
    
    
LcLogDataArray:
LcLogMultiData:
    rts;

/*
 *  NullInitLogChannel
 *
 *  local subfunction to initialize a LogChannel to all 0 or NULL values
 */

NullInitLogChannel:
    AR = 0;
    CNTR = LEN_LOGCHANNEL_T;
    do NILC until CE;
NILC: DM(I1, M1) = AR;

    rts;


SendHSLATECInitSeq:

    AX0 = 0x3300;
    AX1 = 0x9966;
    AY0 = 0xFBFA;
    AY1 = 0xFDFC;
    
    #define HSLATECINITSEQHDR           0x3300
    
    !AR = HSLATECINITSEQHDR;
    !IO(HSL_HEADER_REG) = AR;
    LOG_DATA_REG(AX0);
    LOOPNOP(10);
    LOG_DATA_REG(AX1);
  
    CNTR = 0x3fff;
    do AI1 until CE;
      LOG_DATA_REG(AY0);
      LOOPNOP(10);
      LOG_DATA_REG(AY1);
      LOOPNOP(10);
AI1:  nop;

    rts;

.ENDMOD;
