/*
 *  LogChannel.dsp
 */

.MODULE                                 m_LogChannel;

#include    "SDO.h"
#include    "C:\dspdev\hsl\log API\sw - 218x\HSL TAC 1\Code\inc\LogChannel.h"
#include    "C:\dspdev\hsl\log API\sw - 218x\HSL TAC 1\Code\inc\LogChannelVars.h"
#include    "C:\dspdev\hsl\log API\sw - 218x\HSL TAC 1\Code\inc\HSLDrv.h"

#include    "C:\dspdev\hsl\log API\sw - 218x\HSL TAC 1\Code\inc\NopUtil.h"

.ENTRY      LcInitialize;

.ENTRY      LcLogStructType;
.ENTRY      LcLogModuleVersion;
.ENTRY      LcLogSimpleEvent;

.ENTRY      LcCreate;
.ENTRY      LcLogEvent;
.ENTRY      LcLogDataArray;
.ENTRY      LcLogMultiData;

.ENTRY      SendHSLATECInitSeq;
.ENTRY      LcFail;
.ENTRY      NullInitLogChannel;
.ENTRY      NullInitPszText;
.ENTRY      NullInitPszDataTypeName;

.ENTRY      LcLogDREGs;

.VAR/DM     l_numbytesPszText;
.VAR/DM     l_numbytesPszDataTypeName;
.VAR/DM     l_numbytesAllChars;
.VAR/DM     l_RestructurePszDataTypeName;
.VAR/DM     l_numbytesCharsAndData;

.VAR/DM     l_DREGs[18];

/*******************************************************************************
*
*  LcInitialize
*
*  - check if already called.  if true, exit
*  - init internal log channel
*  - write to global indicating internal channel exists
*
*  registers destroyed
*
*  AR, ASTAT, I1, M1
*
*******************************************************************************/

LcInitialize:
    AR = DM(gboLcInitialized);
    AR = pass AR;
    if EQ jump StartLcInitialize;
    AR = RETCODE_LCINITIALIZE_LC_WAS_INIT;
    rts;

StartLcInitialize:
    AR = LC_TRUE;
    DM(gboLcInitialized) = AR;
   
    LC_IREG0 = ^g_LogChannel;
    LC_MREG0 = 1;
    call NullInitLogChannel;

    AR = LC_TRUE;
    DM(gboLcLogChannelCreate) = AR;
    AR = RETCODE_LCINITIALIZE_SUCCESS;
    rts;

/*******************************************************************************
*
*  LcLogStructType
*
*  pass 4 parameters in g_pp_LcData
*    - generic name for ATEC to display
*    - pointer to data
*    - number of bytes to transfer
*    - name of structure
*
*******************************************************************************/

LcLogStructType:
    call NestedLogCheck;
    AR = pass AR;
    if LT rts;                              /* if nested call, exit */

    AR = DM(gboLcInitialized);
    AR = pass AR;
    if NE jump LcLogStructType_LcInitOK;
    AR = RETCODE_LCLOGSTRUCTTYPE_LC_NOT_INIT;
    LC_RETURN;                              /* reset nested semaphore, rts */

LcLogStructType_LcInitOK:

    AR = LC_FALSE;
    DM(l_RestructurePszDataTypeName) = AR;

    AR = 0;
    AF = pass 0;
    LC_IREG0 = ^g_pp_LcData+OFFSET_LOGSTRUCT_PSZTEXT;
    LC_MREG0 = 1;
    AY0 = 0xff00;
GetPszTextCount:
    AR = DM(LC_IREG0, LC_MREG0);
    AR = pass AR;
    if EQ jump LcLogStructType_StopPszTextCount;
    AF = AF + 1;                            /* at least one char */
    AR = AR and AY0;
    if EQ jump LcLogStructType_StopPszTextCount;
    AF = AF + 1;
    jump GetPszTextCount;
    
LcLogStructType_StopPszTextCount:
    AF = AF + 1;                            /* log API wants one more byte */
    AR = pass AF;
    DM(l_numbytesPszText) = AR;

/*
 *  check if there is an even number of chars counted here.  if so, the NULL
 *  char is in the right place and PszDataTypeName can be written out directly
 *  without any sort of restructuring.  if not, must insert a NULL byte and
 *  shift the entire string by one byte.
 */
 
    AF = tstbit 0 of AR;
    if EQ jump LcLogStructType_StartPszDataTypeNameCount;
    
    AR = LC_TRUE;
    DM(l_RestructurePszDataTypeName) = AR;

LcLogStructType_StartPszDataTypeNameCount:
    LC_IREG0 = ^g_pp_LcData+OFFSET_LOGSTRUCT_PSZDATATYPENAME;
    AR = 0;
    AF = pass 0;
    AY0 = 0xff00;
GetPszDataTypeNameCount:
    AR = DM(LC_IREG0, LC_MREG0);
    AR = pass AR;
    if EQ jump LcLogStructType_StopPszDataTypeNameCount;
    AF = AF + 1;                            /* at least one char */
    AR = AR and AY0;
    if EQ jump LcLogStructType_StopPszDataTypeNameCount;
    AF = AF + 1;
    jump GetPszDataTypeNameCount;


LcLogStructType_StopPszDataTypeNameCount:
    AF = AF + 1;                            /* log API wants one more byte */
    AR = pass AF;
    DM(l_numbytesPszDataTypeName) = AR;
    AY1 = DM(l_numbytesPszText);            /* retrieve len(PszText) */
    AR = AR + AY1;                          /* length of both char arrays */
    
/*
 *  increase total such that it is divisble by 4
 */

    AY1 = 0x0003;
    AR = AR + AY1;
    AY1 = 0xfffc;
    AR = AR and AY1;
    DM(l_numbytesAllChars) = AR;
    
    AR = DM(g_pp_LcData+OFFSET_LOGSTRUCT_DWDATASIZE);
    AX1 = AR;
    AF = tstbit 0 of AR;
    if NE AR = AR + 1;                      /* if odd, add 1 */
    
    AY1 = DM(l_numbytesAllChars);
    AF = AR + AY1;                          /* add numwords + charbytes */
    AR = AR + AF;                           /* double numwords for bytes */
    DM(l_numbytesCharsAndData) = AR;

    /* check if PszDataTypeName needs restructuring */
    AR = DM(l_RestructurePszDataTypeName);
    AR = pass AR;
    if EQ jump LcLogStructType_SendNumBytesAndHeader;

    /* perform restructuring */
    SR0 = DM(l_numbytesPszDataTypeName);
    SR = lshift SR0 by -1 (LO);
    !AR = SR0 - 1;
    CNTR = SR0;
    
    LC_IREG0 = ^g_pp_LcData+OFFSET_LOGSTRUCT_PSZDATATYPENAME;
    LC_MREG1 = 0;
    AR = DM(LC_IREG0, LC_MREG1);
    MR0 = AR;
    SR = lshift AR by 8 (LO);
    DM(LC_IREG0, LC_MREG0) = SR0;
    do BSH until CE;
      SR0 = DM(LC_IREG0, LC_MREG1);
      MR1 = SR0;
      SR = lshift SR0 by 8 (LO);
      AY0 = SR0;
      SR = lshift MR0 by -8 (LO);
      MR0 = MR1;
      AR = SR0 or AY0;
BSH:  DM(LC_IREG0, LC_MREG0) = AR;

/*
 *  we now have enough info to start actual logging process.  after sending the
 *  total number of bytes and the header, must perform check on pData element
 *  to determine if data is in PM/DM and then dump the data as appropriate.
 */

LcLogStructType_SendNumBytesAndHeader:

    /* NumBytes in AR */
    LOG_SINGLE_DM(l_numbytesCharsAndData, AY0);
    LOG_VAL(0x0000, AY0);
    
    /* generate and send header */
    AY0 = ENUM_LCID_LOGSTRUCTTYPE << LC_DATA_POS;
    LOG_DATA_REG(AY0);
    AY0 = 0;
    LOG_DATA_REG(AY0);
    
    /* send PszText */
    LC_IREG0 = ^g_pp_LcData+OFFSET_LOGSTRUCT_PSZTEXT;
    SR0 = DM(l_numbytesPszText);
    SR = lshift SR0 by -1 (LO);             /* len calc'ed in bytes */
    
    /* test if it's odd.  if so, can send normally */
    AR = tstbit 0 of SR0;
    if NE jump LcLogStructType_PszTextDirectWrite;
    
    /* if even, must construct last word with byte from next char */
    !AR = 0x0012;
    !call SDO_WriteValue_;
    
LcLogStructType_PszTextDirectWrite:    
    SR0 = DM(l_numbytesPszText);
    SR = lshift SR0 by -1 (LO);
    CNTR = SR0;
    !CNTR = DM(l_numbytesPszText);
    do SPT until CE;
      AR = DM(LC_IREG0, LC_MREG0);
      LOG_DATA_REG(AR);
SPT:  nop;

#if 0
    /* generic name */
    LOG_VAL(0x6574, AR);                    /* et */
    LOG_VAL(0x706d, AR);                    /* pm */
    LOG_VAL(0x7400, AR);                    /* t\0 */
    
    /* struct name */
    LOG_VAL(0x706d, AR);                    /* mt */
    LOG_VAL(0x0000, AR);                    /* \0p */

    LOG_VAL(0x0000, AR);
#endif 

    /* send PszDataTypeName */
    
    SR0 = DM(l_numbytesPszText);
    AF = pass 0;
    AR = tstbit 0 of SR0;
    if EQ jump LcLogStructType_ShiftNoIncrement;
    SR0 = DM(l_numbytesPszDataTypeName);
    AR = tstbit 0 of SR0;
    if EQ jump LcLogStructType_ShiftNoIncrement;
!w: jump w;
    AF = pass 1;
    

    !AF = pass 0;
LcLogStructType_ShiftNoIncrement:
    SR0 = DM(l_numbytesPszDataTypeName);
    SR = lshift SR0 by -1 (LO);
    AR = SR0 + AF;
    CNTR = AR;
    
    LC_IREG0 = ^g_pp_LcData+OFFSET_LOGSTRUCT_PSZDATATYPENAME;

    !SR = lshift SR0 by -1 (LO);             /* len calc'ed in bytes */
    !AR = SR0;
    !CNTR = AR;
    do SDTN until CE;
      AR = DM(LC_IREG0, LC_MREG0);
      LOG_DATA_REG(AR);
SDTN: nop;

/*
 *  add writes based on last 2 bits in tot num chars
 *
 *   00:  0 0x0000
 *   01:  2 0x0000
 *   10:  1 0x0000
 *   11:  1 0x0000
 */

    AX0 = DM(l_numbytesPszDataTypeName);
    AY0 = DM(l_numbytesPszText);
    AR = AX0 + AY0;
    AY0 = 0x0003;
    AR = AR and AY0;
    LC_P_IREG0 = ^LcLogStructType_AddWriteJT;
    LC_P_LREG0 = 0;
    LC_P_MREG0 = AR;
    modify(LC_P_IREG0, LC_P_MREG0);
    jump (LC_P_IREG0);

LcLogStructType_AddWriteJT:
    jump LcLogStructType_ZeroWrite;
    jump LcLogStructType_TwoWrite;
    jump LcLogStructType_OneWrite;
    jump LcLogStructType_OneWrite;

LcLogStructType_TwoWrite:
    LOG_VAL(0x0000, AY0);
LcLogStructType_OneWrite:
    LOG_VAL(0x0000, AY0);
LcLogStructType_ZeroWrite:

    AX0 = DM(g_pp_LcData+OFFSET_LOGSTRUCT_PDATA);
    AR = tstbit 15 of AX0;
    if EQ jump LcLogStructType_LogIsDM;
    
LcLogStructType_LogIsPM:
    LC_P_IREG0 = AX0;                        /* MSB set, but Ireg is 14 bits */
    LC_P_MREG0 = 1;
    CNTR = AX1;
    do LPM until CE;
      AR = PM(LC_P_IREG0, LC_P_MREG0);
      LOG_DATA_REG(AR);
LPM:  nop;

    /* if odd # words, must write one for byte count to be divisble by 4 */
    AX0 = AY1;
    AR = tstbit 0 of AX1;
    if EQ jump LcLogStructType_Finished;
    
    LOG_VAL(0x0000, AR);

LcLogStructType_LogIsDM:
    LC_IREG0 = AX0;
    CNTR = AX1;
    do LDM until CE;
      AR = DM(LC_IREG0, LC_MREG0);
      LOG_DATA_REG(AR);
LDM:  nop;

    /* if odd # words, must write one for byte count to be divisble by 4 */
    AR = tstbit 0 of AX1;
    if EQ jump LcLogStructType_Finished;
    
    LOG_VAL(0x0000, AR);
  

LcLogStructType_Finished:
    
    /* log complete, exit LcLogStructType */
    AR = RETCODE_LCLOGSTRUCTTYPE_SUCCESS;
    LC_RETURN;                              /* reset nested semaphore, rts */

/*******************************************************************************
*
*  LcLogModuleVersion
*
*******************************************************************************/
    
LcLogModuleVersion:
    call NestedLogCheck;
    AR = pass AR;
    if LT rts;                              /* AR is +/-1 from NestedLogCheck */

    AR = DM(gboLcInitialized);
    AR = pass AR;
    if NE jump LcLogSimpleEvent_LcInitOK;
    AR = RETCODE_LCLOGMODULEVERSION_LC_NOT_INIT;
    LC_RETURN;                              /* reset nested semaphore, rts */

    
    LC_RETURN;                              /* reset nested semaphore, rts */
    
/*******************************************************************************
* 
*  LcLogSimpleEvent
*
*  log a single 32 bit word using the internal log channel
*
*******************************************************************************/

LcLogSimpleEvent:
    call NestedLogCheck;
    AR = pass AR;
    if LT rts;                              /* if nested call, exit */
    
    AR = DM(gboLcInitialized);
    AR = pass AR;
    if NE jump LcLogSimpleEvent_LcInitOK;
    AR = RETCODE_LCLOGSIMPLEEVENT_LC_NOT_INIT;
    LC_RETURN;                              /* reset nested semaphore, rts */

/*
 *  important!  the log API is 32 bit centric.  however, in this case we can
 *  cheat a bit because we know that (ENUM_LCID_LOGSIMPLEEVENT << LC_DATA_POS)
 *  will not extend beyond 16 bits.  therefore the first word written is 0, and
 *  the second word is this header word
 */

LcLogSimpleEvent_LcInitOK:

    /* always sending 4 bytes in simple event */
    LOG_VAL(0x0004, AY0);
    LOG_VAL(0x0000, AY0);

    AY0 = ENUM_LCID_LOGSIMPLEEVENT << LC_DATA_POS;
    LOG_DATA_REG(AY0);
    AY0 = 0;
    LOG_DATA_REG(AY0);
    
    /* all Lc API passed param data stored in g_pp_LcData */
    LOG_SINGLE_DM(g_pp_LcData+0, AY0);
    LOG_SINGLE_DM(g_pp_LcData+1, AY0);
    
    AR = RETCODE_LCLOGSIMPLEEVENT_SUCCESS;
    LC_RETURN;                              /* reset nested semaphore, rts */

/*******************************************************************************
*
*  LcCreate
*
*  create a new application log channel.  parameters are passed in the
*  following global arrays (and therefore must be initialized before the call
*  to this function.
*
*  g_pp_LogChannel
*
*  modified values
*
*    AR, AY0, ASTAT, MY0, MR0, MR1, MR2, MSTAT, M2, I1, I2
*
*  return values
*
*******************************************************************************/

LcCreate:
    call NestedLogCheck;
    AR = pass AR;
    if LT rts;                              /* if nested call, exit */
    
    AR = DM(gboLcInitialized);
    AR = pass AR;
    if NE jump LcCreate_LcInitOK;
    AR = RETCODE_LCCREATE_LC_NOT_INIT;
    LC_RETURN;                              /* reset nested semaphore, rts */
    
LcCreate_LcInitOK:
    AR = DM(g_NextChannelIdToUse);
    AY0 = MAX_APP_LOGCHANNEL;
    NONE = AR - AY0;
    if LE jump LcCreate_LcChanAvailable;
    AR = RETCODE_LCCREATE_NO_CHAN_AVAILABLE;
    LC_RETURN;                              /* reset nested semaphore, rts */

LcCreate_LcChanAvailable:
    AR = DM(g_NextAvailableChannel);
    MY0 = LEN_LOGCHANNEL;
    LC_IREG0 = mstat;
    ena m_mode;
    MR = AR * MY0 (UU);
    mstat = I1;
    AY0 = ^g_LogChannel;
    AR = MR0 + AY0;
    LC_IREG0 = AR;
    MR0 = I1;                               /* save head ptr to LogChannel */
    LC_IREG1 = ^g_pp_LcData;
    LC_MREG0 = 1;
    CNTR = LEN_LOGCHANNEL;
    do LCC1 until CE;
      AR = DM(LC_IREG1, LC_MREG0);
LCC1: DM(LC_IREG0, LC_MREG0) = AR;
    
    AY0 = OFFSET_LOGCHANNEL_ID;
    AR = MR0 + AY0;
    LC_IREG1 = AR;                          /* I2 = g_LogChannel.ID */
    AR = DM(g_NextChannelIdToUse);
    DM(LC_IREG1, LC_MREG0) = AR;            /* channel ID to g_LogChannel */
    AR = AR + 1;
    DM(g_NextChannelIdToUse) = AR;
    
    /* separate from g_NextChannelIdToUse, guaranteed to start at 1 */
    AR = DM(g_NextAvailableChannel);
    AR = AR + 1;
    DM(g_NextAvailableChannel) = AR;
    
/*
 *  log the creation of an app channel.  since the max length of the name of the
 *  channel is known, it is easier to transmit the entire LOGCHANNEL struct
 *  rather than extract the exact length of the string.
 */

    /* sending entire LOGCHANNEL struct */
    LOG_VAL(LENBYTES_LOGCHANNEL, AY0);
    LOG_VAL(0x0000, AY0);

    /* LcCreate header message */
    AY0 = ENUM_LCID_CREATE << LC_DATA_POS;
    LOG_DATA_REG(AY0);
    AY0 = 0;
    LOG_DATA_REG(AY0);

    /* send ID and options passed to LcCreate - MR0 ptr to head of g_LogChannel */
    AY0 = OFFSET_LOGCHANNEL_ID;
    AR = MR0 + AY0;
    LC_IREG0 = AR;
    AR = DM(LC_IREG0, LC_MREG0);
    LOG_DATA_REG(AR);

    AY0 = OFFSET_LOGCHANNEL_OPTS;
    AR = MR0 + AY0;
    LC_IREG0 = AR;
    AR = DM(LC_IREG0, LC_MREG0);
    LOG_DATA_REG(AR);

    /* send entire char array */
    CNTR = LEN_LOGCHANNEL_NAME;
    AY0 = OFFSET_LOGCHANNEL_NAME;
    AR = MR0 + AY0;
    LC_IREG0 = AR;
    do SCN until CE;
      AY0 = DM(LC_IREG0, LC_MREG0);
      LOG_DATA_REG(AY0);
SCN:  nop;
  
    AR = RETCODE_LCCREATE_SUCCESS;
    LC_RETURN;                              /* reset nested semaphore, rts */
    

/*******************************************************************************
*
*  LcLogEvent
*
*******************************************************************************/

LcLogEvent:
    call NestedLogCheck;
    AR = pass AR;
    if LT rts;                              /* if nested call, exit */
    
    AR = DM(gboLcInitialized);
    AR = pass AR;
    if NE jump LcLogEvent_LcInitOK;
    AR = RETCODE_LCLOGEVENT_LC_NOT_INIT;
    LC_RETURN;                              /* reset nested semaphore, rts */
    
LcLogEvent_LcInitOK:
    LC_RETURN;                              /* reset nested semaphore, rts */


/*******************************************************************************
*
*  LcLogDataArray
*
*******************************************************************************/
    
LcLogDataArray:
    call NestedLogCheck;
    AR = pass AR;
    if LT rts;                              /* if nested call, exit */
    
    LC_RETURN;                              /* reset nested semaphore, rts */

    
/*******************************************************************************
*
*  LcLogMultiData
*
*******************************************************************************/

LcLogMultiData:
    call NestedLogCheck;
    AR = pass AR;
    if LT rts;                              /* if nested call, exit */
    
    LC_RETURN;                              /* reset nested semaphore, rts */

/*******************************************************************************
*
*  global functions which are not a part of the Lc API specification
*
*******************************************************************************/

/*
 *  NullInitLogChannel
 *
 *  local subfunction to initialize a LogChannel to all 0 or NULL values
 */

NullInitLogChannel:
    AR = 0;
    CNTR = LEN_LOGCHANNEL;
    do NILC until CE;
NILC: DM(LC_IREG0, LC_MREG0) = AR;

    rts;

NullInitPszText:
    AR = 0;
    LC_IREG0 = ^g_pp_LcData+OFFSET_LOGSTRUCT_PSZTEXT;
    LC_MREG0 = 1;
    CNTR = LEN_LOGSTRUCT_PSZTEXT;
    do NIPT until CE;
NIPT: DM(LC_IREG0, LC_MREG0) = AR;

    rts;

NullInitPszDataTypeName:
    AR = 0;
    LC_IREG0 = ^g_pp_LcData+OFFSET_LOGSTRUCT_PSZDATATYPENAME;
    LC_MREG0 = 1;
    CNTR = LEN_LOGSTRUCT_PSZDATATYPENAME;
    do NIPD until CE;
NIPD: DM(LC_IREG0, LC_MREG0) = AR;

    rts;

/*
 *  SendHSLATECInitSeq
 *  
 *  this function must be called after the HSL is initialized (via calling
 *  HSLInit) to signal to ATEC to start listening to signals sent from the HSL.
 *  note that this is the only place that the header register is written to by
 *  the LC API.  all subsequent writes are made to the data register (ie the
 *  log single and log last data register aren't used by this API.
 */

SendHSLATECInitSeq:

    /* only usage of header register - HSL_DATA_REG used from now on */
    AX0 = 0x3300;
    LOG_HEADER_REG(AX0);
    
    AX0 = 0x9966;
    LOG_DATA_REG(AX0);
    
    AX0 = 0xFBFA;
    AX1 = 0xFDFC; 
    
    CNTR = 0x3fff;
    do AI1 until CE;
      LOG_DATA_REG(AX0);
      LOG_DATA_REG(AX1);
AI1:  nop;

/*
 *  sequence to dump known data to HSL and force ATEC to respond to individual
 *  commands.  if this is not done, ATEC will not respond to short commands
 *  until the HW HSL buffer is full.  note that this code does not use the LC
 *  API because we can't guarantee that it has been initialized at this point.
 */

    AX0 = L_ASCII_a;
    AY0 = U_ASCII_a;

    /* write aaaa through zzzz four times */
    CNTR = 26;
    do LV until ce;
      CNTR = 4;
      do LV2 until CE;
        AR = AX0 or AY0;
        LOG_VAL(0x0004, AY1);               /* hard-coded event constant */
        LOG_VAL(0x0000, AY1);               /* hard-coded event constant */
        LOG_VAL(0xa000, AY1);               /* hard-coded event constant */
        LOG_VAL(0x0000, AY1);               /* hard-coded event constant */
        LOG_DATA_REG(AR);
        LOG_DATA_REG(AR);
LV2:    nop;
      AR = AX0;
      AR = AR + 1;
      AX0 = AR;
      SR = lshift AR by 8 (LO);
LV:   AY0 = SR0;
          
    rts;

/*******************************************************************************
*
*  NestedLogCheck, EndOfLcCall
*
*  at the start of any logging function, code must check if another logging
*  function is happening in a lower context.  the log API is not re-entrant
*  and must be protected from nested calls.  NestedLogCheck performs this check,
*  while EndOfLcCall sets g_NestedLogStatus back to false and returns back to
*  mainline code.
*
*******************************************************************************/

NestedLogCheck:
    AR = DM(g_NestedLogStatus);
    AR = pass AR;
    if EQ jump LogNotNested;
    
LogNested:
    AR = RETCODE_NESTED_LOG;
    rts;

LogNotNested:
    AR = LC_TRUE;
    DM(g_NestedLogStatus) = AR;
    rts;

EndOfLcCall:
    AY0 = LC_FALSE;
    DM(g_NestedLogStatus) = AY0;
    rts;

/*
 *  just a place to trap if any Lc functions fail
 */

LcFail:
    jump LcFail;

/*******************************************************************************
*
*  LcLogDREGs
*
*  MSP430-specific function, added to Lc API specification because all coding
*  happens in ASM for this platform, where MSP500 is primarily concerned with
*  logging variables and data.
*
*  modified registers:
*
*  AR, ASTAT, AY0
*
*******************************************************************************/

LcLogDREGs:
    LC_IREG0 = ^l_DREGs;
    LC_LREG0 = 0;
    LC_MREG0 = 1;
    DM(LC_IREG0, LC_MREG0) = AR;
    DM(LC_IREG0, LC_MREG0) = AX0;
    DM(LC_IREG0, LC_MREG0) = AX1;
    DM(LC_IREG0, LC_MREG0) = AY0;
    DM(LC_IREG0, LC_MREG0) = AY1;
    AR = pass AF;
    DM(LC_IREG0, LC_MREG0) = AR;
    DM(LC_IREG0, LC_MREG0) = MR0;
    DM(LC_IREG0, LC_MREG0) = MR1;
    DM(LC_IREG0, LC_MREG0) = MR2;
    DM(LC_IREG0, LC_MREG0) = MX0;
    DM(LC_IREG0, LC_MREG0) = MX1;
    DM(LC_IREG0, LC_MREG0) = MY0;
    DM(LC_IREG0, LC_MREG0) = MY1;
    DM(LC_IREG0, LC_MREG0) = SR0;
    DM(LC_IREG0, LC_MREG0) = SR1;
    DM(LC_IREG0, LC_MREG0) = SI;
    DM(LC_IREG0, LC_MREG0) = SE;
    AR = SB;
    DM(LC_IREG0, LC_MREG0) = AR;
    
    LC_INIT_LST_PSZTEXT_6(_L_, _o_, _g_, _R_, _e_, _g_, AY0);
    LC_INIT_LST_PDATA(LOGSTRUCT_PDATA_DM, ^l_DREGs);
    LC_INIT_LST_DWDATASIZE(%l_DREGs, AR);
    LC_INIT_LST_PSZDATATYPENAME_5(_D_, _R_, _E_, _G_, _s_, AY0);
    call LcLogStructType;
    LC_JUMP_ON_FAIL(LcFail);
    
    rts;

.ENDMOD;
