/***************************************************************************
 *             
 *   This file contains Analog Devices, Inc. proprietary information.
 *   Copyright 2001 by Analog Devices, Inc.  All rights reserved.
 *
 *    File           : tacDisplay7Seg.dsp
 *    Project        : DSP TAC
 *    Author(s)      : Raheem Baig
 *    Started        : 3rd July 2004
 *    Description    : DSP TAC displays the TCS message bytes received on 
 *                     seven segment of ANVIL EVB.
 *
 *    Revision Control system (RCS) Information:
 *
 *    $Workfile:$
 *    $Revision:$
 *     $Modtime:$
 *        $Date:$
 *      $Author:$
 *     $Archive:$
 *  
 ***************************************************************************/

.MODULE  TAC_7SEG_DISPLAY;

/*
 * includes
 */
#include "C:\dspdev\klio\context save and restore\Code\inc\tic_drv.h"          /* TIC driver header file */
#include "SDO.h"
#include "C:\dspdev\klio\context save and restore\Code\inc\NopUtil.h"
#include "C:\dspdev\klio\context save and restore\Code\inc\HSLDrv.h"

#include "C:\dspdev\klio\context save and restore\Code\inc\LogChannelFuncs.h"

#include "C:\dspdev\klio\context save and restore\Code\inc\LogChannel.h"
#include "C:\dspdev\klio\context save and restore\Code\inc\LogChannelVars.h"


/*
 * Variables and Externals
 */

.VAR/DM StructTestData[6];
.GLOBAL StructTestData;
.INIT   StructTestData:         1111, 2222, 3333, 4444, 5555, 6666;

/*
 * Entries 
 */
.ENTRY  TacInit_;        /* Subroutine to initialize TAC */
.ENTRY  TacMain_;        /* Subroutine to execute TAC */
.ENTRY  TacDelay_;       /* delay function */
.ENTRY  TacSendTcsMsg_;  /* subroutine to send response TCS message */


.external  CSRInit, CSRTest;
/**************************************************************************
 **************************************************************************/
TacInit_:

   nop;nop;nop;nop;

RTS;
/**************************************************************************
 **************************************************************************/
TacMain_:

/*
 *  start main code here
 */

    call CSRInit;
    call CSRTest;

wait:
    jump wait;


!#ifdef DSP_SIMULATION
    AR = TRUE;
!#else
   !AR = DM(newTcsMsgFlag);
!#endif
   !AY0 = TRUE;
   !AR = AR - AY0;

   !IF NE JUMP returnTacMain;   
.var temp;
    call USC_Config_;
    !PMOVLAY = 8;
    AR = 0x1111;
    call SDO_WriteValue_;

    
    call InitHSL;
    AR = 0x2222;
    call SDO_WriteValue_;

    !call WeirdHSLProblem;

    
    call SendHSLATECInitSeq;
    AR = 0x2222;
    call SDO_WriteValue_;
     

    call LcInitialize;
    LC_JUMP_ON_FAIL(LcFail);

    AR = 0x3333;
    call SDO_WriteValue_;

!#ifdef DSP_SIMULATION    
here:
    call Regress;
    AR = 0x4444;
    call SDO_WriteValue_;

!#endif

w: jump w;

    !call LcLogDREGs;
    
    
    /* reached end of tests, write 0x0001 to 7 seg and exit */
    .var count;
    AR = DM(count);
    AR = AR + 1;
    DM(count) = AR;
    !AR = 0x001;
    !call SDO_WriteValue_;

    !jump here;

send_resp_msg:

    call TacSendTcsMsg_; /* Send TCS response message as it was received
                         */

clr_new_msg_flag:
    AR = FALSE;    
    DM(newTcsMsgFlag) = AR;

clear_sts_flag:
    /*
     * CLEAR STS (MCU2DSP TCS messaging) flag, 
     * to receive a new TCS msg from MCU 
     */
    call TicClrMCU2DSPStsFlag_;

returnTacMain:
   RTS;
/**************************************************************************
 **************************************************************************/
/*
 **************************************************************************
 * Function Name: TacDelay_
 * Parameters: None
 * Return: None
 * Side effects: delay the execution by some time depening on the loop counter.
 *
 **************************************************************************
 */
   
TacDelay_:

.VAR/DM/RAM save_AR_tacDelay;

    DM(save_AR_tacDelay) = AR;

    CNTR = 0x1ff;
    DO outer_loop_tacDelay UNTIL CE;

    AR = 0x7FF;
    nop;nop;nop;nop;nop;nop;nop;

wait_here_tacDelay:
    AR = AR - 1;

    IF NE JUMP wait_here_tacDelay;

    nop;nop;nop;nop;nop;nop;nop;

outer_loop_tacDelay: nop;

    AR = DM(save_AR_tacDelay);

    RTS;
/**************************************************************************
 **************************************************************************/
/*
 **************************************************************************
 * Function Name: TacSendTcsMsg
 * Parameters: None
 * Return: None
 * Side effects: This function write the new TCS message to the MCU-DSP shared
 * memory area2(where ARM reads) and sets the status flag 2(which is cleared by
 * ARM).
 *
 * function to send back the TCS msg, which is received, to MCU. 
 * 
 **************************************************************************
 */
TacSendTcsMsg_:
	L0 = 0;
	L1 = 0;
	M1 = 1;

    /* copy the TCS message received to transmit as a test vector */    
    I1 = ^ticRxTcsMsg + OFFSET_TCS_MSG_LEN;
	AR = DM(I1,M1);
	CNTR = AR; /* size of TCS msg */ 
	I0 = ^ticRxTcsMsg + OFFSET_TCS_MSG_BODY;
	I1 = ^ticTxTcsMsg + OFFSET_TCS_MSG_BODY;
	
    DO copy_tcs_msg UNTIL CE;
	AR = DM(I0,M1);
copy_tcs_msg:	DM(I1,M1) = AR;

    call TicSendTcsMsg2MCU_;

    RTS;   /* return of TacSendTcsMsg_() */

HSLIncTest:

    AR = 0;
    AY0 = 0xabcd;
    AY1 = 0x8000;
    AX0 = 0;
    
    dm(temp) = ar;
    CNTR = 0x800;
    do LL until CE;
      AY0 = 0xabcd;
      LOG_HEADER_REG(AY0);
      !LOG_LAST_REG(AX0);
      !LOG_SINGLE_DM(temp, AR);
      LOG_LAST_DM(temp, AR);
      AR = AR + 1;
      dm(temp) = ar;
      call SDO_WriteValue_;
      !call TacDelay_;
      !ay0 = 0xabcd;
      !LOG_DATA_REG(AY0);
LL:   nop;

    rts;

Regress:

/*
 *  various simple events
 */

    LC_INIT_SIMPLE_EVENT_NAME(_W_, _H_, _E_, _E_, AY0);
    call LcLogSimpleEvent;
    LC_JUMP_ON_FAIL(LcFail);
    
    LC_INIT_SIMPLE_EVENT_NAME(_I_, _NULL_, _NULL_, _NULL_, AY0);
    call LcLogSimpleEvent;
    LC_JUMP_ON_FAIL(LcFail);
    
    LC_INIT_SIMPLE_EVENT_NAME(_C_, _A_, _N_, _NULL_, AY0);
    call LcLogSimpleEvent;
    LC_JUMP_ON_FAIL(LcFail);
    
    LC_INIT_SIMPLE_EVENT_NAME(_N_, _O_, _W_, _NULL_, AY0);
    call LcLogSimpleEvent;
    LC_JUMP_ON_FAIL(LcFail);
    
    LC_INIT_SIMPLE_EVENT_NAME(_L_, _O_, _G_, _NULL_, AY0);
    call LcLogSimpleEvent;
    LC_JUMP_ON_FAIL(LcFail);

/*
 *  LcCreate tests; application channels of various lengths
 */

    /* create an app event channel 'ABCD' */
    LC_INIT_PP_LOGCHANNEL_NAME_4(_A_, _B_, _C_, _D_, AY0);
    LC_INIT_PP_LOGCHANNEL_OPTS(LC_FLAGS_EVENT, AY0);
    call LcCreate;
    LC_JUMP_ON_FAIL(LcFail);

    LC_INIT_PP_LOGCHANNEL_NAME_5(_C_, _H_, _A_, _R_, _5_, AY0);
    LC_INIT_PP_LOGCHANNEL_OPTS(LC_FLAGS_EVENT, AY0);
    call LcCreate;
    LC_JUMP_ON_FAIL(LcFail);

/*
 *  LcLogStructType tests.  a good start for full coverage comes from testing
 *  all 16 combinations of byte lengths {0, 0}... {3, 0}... {0, 3}... {3, 3}.
 *  all working LcLogStructTypes put in here.
 */

    
    LC_INIT_SIMPLE_EVENT_NAME(_S_, _R_, _e_, _g_, AY0);
    call LcLogSimpleEvent;
    LC_JUMP_ON_FAIL(LcFail);

/*******************************************************************************
*
*  {0, 0}... {3, 0}
*
*******************************************************************************/    
    
    LC_INIT_LST_PSZTEXT_4(_t_, _e_, _m_, _p_, AY0);
    LC_INIT_LST_PDATA(LOGSTRUCT_PDATA_DM, ^StructTestData);
    LC_INIT_LST_DWDATASIZE(%StructTestData, AY0);
    LC_INIT_LST_PSZDATATYPENAME_4(_t_, _m_, _p_, _a_, AY0);
    call LcLogStructType;
    LC_JUMP_ON_FAIL(LcFail);
    
    LC_INIT_LST_PSZTEXT_5(_t_, _e_, _m_, _p_, _a_, AY0);
    LC_INIT_LST_PDATA(LOGSTRUCT_PDATA_DM, ^StructTestData);
    LC_INIT_LST_DWDATASIZE(%StructTestData, AY0);
    LC_INIT_LST_PSZDATATYPENAME_4(_t_, _m_, _p_, _a_, AY0);
    call LcLogStructType;
    LC_JUMP_ON_FAIL(LcFail);
    
    LC_INIT_LST_PSZTEXT_6(_t_, _e_, _m_, _p_, _a_, _b_, AY0);
    LC_INIT_LST_PDATA(LOGSTRUCT_PDATA_DM, ^StructTestData);
    LC_INIT_LST_DWDATASIZE(%StructTestData, AY0);
    LC_INIT_LST_PSZDATATYPENAME_4(_t_, _m_, _p_, _a_, AY0);
    call LcLogStructType;
    LC_JUMP_ON_FAIL(LcFail);
        
    LC_INIT_LST_PSZTEXT_3(_t_, _e_, _m_, AY0);
    LC_INIT_LST_PDATA(LOGSTRUCT_PDATA_DM, ^StructTestData);
    LC_INIT_LST_DWDATASIZE(%StructTestData, AY0);
    LC_INIT_LST_PSZDATATYPENAME_4(_t_, _m_, _p_, _a_, AY0);
    call LcLogStructType;
    LC_JUMP_ON_FAIL(LcFail);

/*******************************************************************************
*
*  {0, 1}... {3, 1}
*
*******************************************************************************/

    LC_INIT_LST_PSZTEXT_4(_t_, _e_, _m_, _p_, AY0);
    LC_INIT_LST_PDATA(LOGSTRUCT_PDATA_DM, ^StructTestData);
    LC_INIT_LST_DWDATASIZE(%StructTestData, AY0);
    LC_INIT_LST_PSZDATATYPENAME_5(_t_, _m_, _p_, _a_, _b_, AY0);
    call LcLogStructType;
    LC_JUMP_ON_FAIL(LcFail);

    LC_INIT_LST_PSZTEXT_5(_t_, _e_, _m_, _p_, _a_, AY0);
    LC_INIT_LST_PDATA(LOGSTRUCT_PDATA_DM, ^StructTestData);
    LC_INIT_LST_DWDATASIZE(%StructTestData, AY0);
    LC_INIT_LST_PSZDATATYPENAME_5(_t_, _m_, _p_, _a_, _b_, AY0);
    call LcLogStructType;
    LC_JUMP_ON_FAIL(LcFail);

    LC_INIT_LST_PSZTEXT_6(_t_, _e_, _m_, _p_, _a_, _b_, AY0);
    LC_INIT_LST_PDATA(LOGSTRUCT_PDATA_DM, ^StructTestData);
    LC_INIT_LST_DWDATASIZE(%StructTestData, AY0);
    LC_INIT_LST_PSZDATATYPENAME_5(_t_, _m_, _p_, _a_, _b_, AY0);
    call LcLogStructType;
    LC_JUMP_ON_FAIL(LcFail);

    LC_INIT_LST_PSZTEXT_3(_t_, _e_, _m_, AY0);
    LC_INIT_LST_PDATA(LOGSTRUCT_PDATA_DM, ^StructTestData);
    LC_INIT_LST_DWDATASIZE(%StructTestData, AY0);
    LC_INIT_LST_PSZDATATYPENAME_5(_t_, _m_, _p_, _a_, _b_, AY0);
    call LcLogStructType;
    LC_JUMP_ON_FAIL(LcFail);
    
/*******************************************************************************
*
*  {0, 2}... {3, 2}
*
*******************************************************************************/

    LC_INIT_LST_PSZTEXT_4(_t_, _e_, _m_, _p_, AY0);
    LC_INIT_LST_PDATA(LOGSTRUCT_PDATA_DM, ^StructTestData);
    LC_INIT_LST_DWDATASIZE(%StructTestData, AY0);
    LC_INIT_LST_PSZDATATYPENAME_6(_t_, _m_, _p_, _a_, _b_, _c_, AY0);
    call LcLogStructType;
    LC_JUMP_ON_FAIL(LcFail);
        
    LC_INIT_LST_PSZTEXT_5(_t_, _e_, _m_, _p_, _a_, AY0);
    LC_INIT_LST_PDATA(LOGSTRUCT_PDATA_DM, ^StructTestData);
    LC_INIT_LST_DWDATASIZE(%StructTestData, AY0);
    LC_INIT_LST_PSZDATATYPENAME_6(_t_, _m_, _p_, _a_, _b_, _c_, AY0);
    call LcLogStructType;
    LC_JUMP_ON_FAIL(LcFail);
    
    LC_INIT_LST_PSZTEXT_6(_t_, _e_, _m_, _p_, _a_, _b_, AY0);
    LC_INIT_LST_PDATA(LOGSTRUCT_PDATA_DM, ^StructTestData);
    LC_INIT_LST_DWDATASIZE(%StructTestData, AY0);
    LC_INIT_LST_PSZDATATYPENAME_6(_t_, _m_, _p_, _a_, _b_, _c_, AY0);
    call LcLogStructType;
    LC_JUMP_ON_FAIL(LcFail);
    
    LC_INIT_LST_PSZTEXT_3(_t_, _e_, _m_, AY0);
    LC_INIT_LST_PDATA(LOGSTRUCT_PDATA_DM, ^StructTestData);
    LC_INIT_LST_DWDATASIZE(%StructTestData, AY0);
    LC_INIT_LST_PSZDATATYPENAME_6(_t_, _m_, _p_, _a_, _b_, _c_, AY0);
    call LcLogStructType;
    LC_JUMP_ON_FAIL(LcFail);
    
/*******************************************************************************
*
*  {0, 3}... {3, 3}
*
*******************************************************************************/

    LC_INIT_LST_PSZTEXT_4(_t_, _e_, _m_, _p_, AY0);
    LC_INIT_LST_PDATA(LOGSTRUCT_PDATA_DM, ^StructTestData);
    LC_INIT_LST_DWDATASIZE(%StructTestData, AY0);
    LC_INIT_LST_PSZDATATYPENAME_3(_t_, _m_, _p_, AY0);
    call LcLogStructType;
    LC_JUMP_ON_FAIL(LcFail);

    LC_INIT_LST_PSZTEXT_5(_t_, _e_, _m_, _p_, _a_, AY0);
    LC_INIT_LST_PDATA(LOGSTRUCT_PDATA_DM, ^StructTestData);
    LC_INIT_LST_DWDATASIZE(%StructTestData, AY0);
    LC_INIT_LST_PSZDATATYPENAME_3(_t_, _m_, _p_, AY0);
    call LcLogStructType;
    LC_JUMP_ON_FAIL(LcFail);

    LC_INIT_LST_PSZTEXT_6(_t_, _e_, _m_, _p_, _a_, _b_, AY0);
    LC_INIT_LST_PDATA(LOGSTRUCT_PDATA_DM, ^StructTestData);
    LC_INIT_LST_DWDATASIZE(%StructTestData, AY0);
    LC_INIT_LST_PSZDATATYPENAME_3(_t_, _m_, _p_, AY0);
    call LcLogStructType;
    LC_JUMP_ON_FAIL(LcFail);

    LC_INIT_LST_PSZTEXT_3(_t_, _e_, _m_, AY0);
    LC_INIT_LST_PDATA(LOGSTRUCT_PDATA_DM, ^StructTestData);
    LC_INIT_LST_DWDATASIZE(%StructTestData, AY0);
    LC_INIT_LST_PSZDATATYPENAME_3(_t_, _m_, _p_, AY0);
    call LcLogStructType;
    LC_JUMP_ON_FAIL(LcFail);

/*
 *  LcLogDREGs test
 */

    AR = 0x0005;
    AF = pass AR;
    AR = 0x0011;
    SB = AR;
    AR = 0x0000;
    AX0 = 0x0001;
    AX1 = 0x0002;
    AY0 = 0x0003;
    AY1 = 0x0004;
    MR0 = 0x0006;
    MR1 = 0x0007;
    MR2 = 0x0008;
    MX0 = 0x0009;
    MX1 = 0x000a;
    MY0 = 0x000b;
    MY1 = 0x000c;
    SR0 = 0x000d;
    SR1 = 0x000e;
    SI = 0x000f;
    SE = 0x0010;

    call LcLogDREGs;
   
    rts;

/**************************************************************************
 **************************************************************************/

.var save_SR0_USC_Config;
.var save_SR1_USC_Config;


USC_Config_:

	M1 = 1;
    /*The address USC SEL LO and HI Registers are 0x80380008 
    			and 0x8038000A resp. on PBUS*/
    SR0 = 0x0008;
    SR1 = 0x8038;
 
    .VAR save_hw_ticGetDspPageReg;
    DM(save_hw_ticGetDspPageReg) = SR1;

    /* Mask lower 11 bits of lower word of Page register */
    SR = LSHIFT SR0 by 5(lo);   		
    SR = LSHIFT SR1 by -5(hi);  		/* place OFFSET(4:0) at bits[15:11] */

    SR1 = DM(save_hw_ticGetDspPageReg);

    DM(RWSM_DM_page10) = SR0;
    DM(RWSM_DM_page11) = SR1;
    
    SR1 = RWSM_DM_page1_ID;
    SR0 = 0x0008;
	
    .VAR save_pageRegId_ticSetDspPmAddr;
	DM(save_pageRegId_ticSetDspPmAddr) = SR1;

    /* get lower word of Page register */
    SR = LSHIFT SR0 by 5(lo);  	/* mask upper 5 bits in lower word of DSPBUS */
    SR = LSHIFT SR0 by -6(lo); 			/* mask lsb in lower word of DSPBUS */
	SR1 = DM(save_pageRegId_ticSetDspPmAddr);
	SR = SR OR LSHIFT SR1 by 10(lo);  	/* place page reg ID in 12:10 bits */
    AR = SR0;  					/* store DSP 14 bit address of DM Ext RAM */
    
	I0 = AR;
	DM(I0,M1) = 0x8888;          /*Low Register*/
	
	DM(I0,M1) = 0x0088;          /* High Register*/

    /*The address USC ENA IN Register is 0x80380010 on PBUS*/
    SR0 = 0x0010;
    SR1 = 0x8038;
 
    DM(save_hw_ticGetDspPageReg) = SR1;

    /* Mask lower 11 bits of lower word of Page register */
    SR = LSHIFT SR0 by 5(lo);   		
    SR = LSHIFT SR1 by -5(hi);  		/* place OFFSET(4:0) at bits[15:11] */

    /* return Page register values */
    SR1 = DM(save_hw_ticGetDspPageReg);


    DM(RWSM_DM_page10) = SR0;
    DM(RWSM_DM_page11) = SR1;
	
    SR1 = RWSM_DM_page1_ID;
    SR0 = 0x0010;
	
 	DM(save_pageRegId_ticSetDspPmAddr) = SR1;

    /* get lower word of Page register */
    SR = LSHIFT SR0 by 5(lo);  	/* mask upper 5 bits in lower word of DSPBUS */
    SR = LSHIFT SR0 by -6(lo); 			/* mask lsb in lower word of DSPBUS */
	SR1 = DM(save_pageRegId_ticSetDspPmAddr);
	SR = SR OR LSHIFT SR1 by 10(lo);  	/* place page reg ID in 12:10 bits */
    AR = SR0;  					/* store DSP 14 bit address of DM Ext RAM */


	I0 = AR;
	DM(I0,M0) = 0x0000;
	
	RTS;

WeirdHSLProblem:

here2:
    PMOVLAY = 8;

    !LG_WRITE_HEADER (ay0);
    ay0 = 0x421;
    DM(0x42) = ay0;
    cntr = 100;
    do lp1 until ce;
lp1: nop;
  
    !LG_WRITE_BODY (ay0);
    ay0 = 1;
    DM(0x41) = ay0;
    cntr = 100;
    do lp2 until ce;
lp2: nop;

    !LG_WRITE_LAST (ar);  
    ar = 5;
    DM(0x40) = ar;

    cntr = 1000;
    do lp3 until ce;
lp3: nop;
  
    PMOVLAY = 0;
    
    jump here2;
    
.ENDMOD;

