/***************************************************************************
 *             
 *   This file contains Analog Devices, Inc. proprietary information.
 *   Copyright 2001 by Analog Devices, Inc.  All rights reserved.
 *
 *    File           : tacDisplay7Seg.dsp
 *    Project        : DSP TAC
 *    Author(s)      : Raheem Baig
 *    Started        : 3rd July 2004
 *    Description    : DSP TAC displays the TCS message bytes received on 
 *                     seven segment of ANVIL EVB.
 *
 *    Revision Control system (RCS) Information:
 *
 *    $Workfile:$
 *    $Revision:$
 *     $Modtime:$
 *        $Date:$
 *      $Author:$
 *     $Archive:$
 *  
 ***************************************************************************/

.MODULE  TAC_7SEG_DISPLAY;

/*
 * includes
 */
#include "tic_drv.h"          /* TIC driver header file */
#include "SDO.h"
#include "..\Code\inc\NopUtil.h"
#include "..\Code\inc\HSLDrv.h"

/*
 * Variables and Externals
 */

/*
 * Entries 
 */
.ENTRY  TacInit_;        /* Subroutine to initialize TAC */
.ENTRY  TacMain_;        /* Subroutine to execute TAC */
.ENTRY  TacDelay_;       /* delay function */
.ENTRY  TacSendTcsMsg_;  /* subroutine to send response TCS message */
/**************************************************************************
 **************************************************************************/
TacInit_:

   nop;nop;nop;nop;

RTS;
/**************************************************************************
 **************************************************************************/
TacMain_:

.VAR/DM/RAM  numBytes;

   AR = DM(newTcsMsgFlag);
   AY0 = TRUE;
   AR = AR - AY0;

   IF NE JUMP returnTacMain;
   
.var temp;

    call InitHSL;

    !call SendHSLATECInitSeq;

    AR = 0;
    AY0 = 0xabcd;
    
    dm(temp) = ar;
    CNTR = 0x800;
    do LL until CE;
      LOG_SINGLE_DM(temp, AR);
      AR = AR + 1;
      dm(temp) = ar;
      LOOPNOP(100);
      call SDO_WriteValue_;
      call TacDelay_;
      ay0 = 0xabcd;
      LOG_REG(AY0);
      LOOPNOP(100);
LL:   nop;

send_resp_msg:

    call TacSendTcsMsg_; /* Send TCS response message as it was received
                         */

clr_new_msg_flag:
    AR = FALSE;    
    DM(newTcsMsgFlag) = AR;

clear_sts_flag:
    /*
     * CLEAR STS (MCU2DSP TCS messaging) flag, 
     * to receive a new TCS msg from MCU 
     */
    call TicClrMCU2DSPStsFlag_;

returnTacMain:
   RTS;
/**************************************************************************
 **************************************************************************/
/*
 **************************************************************************
 * Function Name: TacDelay_
 * Parameters: None
 * Return: None
 * Side effects: delay the execution by some time depening on the loop counter.
 *
 **************************************************************************
 */
   
TacDelay_:

.VAR/DM/RAM save_AR_tacDelay;

    DM(save_AR_tacDelay) = AR;

    CNTR = 0x1ff;
    DO outer_loop_tacDelay UNTIL CE;

    AR = 0x7FF;
    nop;nop;nop;nop;nop;nop;nop;

wait_here_tacDelay:
    AR = AR - 1;

    IF NE JUMP wait_here_tacDelay;

    nop;nop;nop;nop;nop;nop;nop;

outer_loop_tacDelay: nop;

    AR = DM(save_AR_tacDelay);

    RTS;
/**************************************************************************
 **************************************************************************/
/*
 **************************************************************************
 * Function Name: TacSendTcsMsg
 * Parameters: None
 * Return: None
 * Side effects: This function write the new TCS message to the MCU-DSP shared
 * memory area2(where ARM reads) and sets the status flag 2(which is cleared by
 * ARM).
 *
 * function to send back the TCS msg, which is received, to MCU. 
 * 
 **************************************************************************
 */
TacSendTcsMsg_:
	L0 = 0;
	L1 = 0;
	M1 = 1;

    /* copy the TCS message received to transmit as a test vector */    
    I1 = ^ticRxTcsMsg + OFFSET_TCS_MSG_LEN;
	AR = DM(I1,M1);
	CNTR = AR; /* size of TCS msg */ 
	I0 = ^ticRxTcsMsg + OFFSET_TCS_MSG_BODY;
	I1 = ^ticTxTcsMsg + OFFSET_TCS_MSG_BODY;
	
    DO copy_tcs_msg UNTIL CE;
	AR = DM(I0,M1);
copy_tcs_msg:	DM(I1,M1) = AR;

    call TicSendTcsMsg2MCU_;

    RTS;   /* return of TacSendTcsMsg_() */


SendHSLATECInitSeq:

    AX0 = 0x3300;
    AX1 = 0x9966;
    AY0 = 0xFBFA;
    AY1 = 0xFDFC;
    
    LOG_REG(AX0);
    LOOPNOP(100);
    LOG_REG(AX1);
    LOOPNOP(100);
    
    CNTR = 0x3fff;
    do AI1 until CE;
      LOG_REG(AY0);
      LOOPNOP(100);
      LOG_REG(AY1);
      LOOPNOP(100);
AI1:  nop;

    rts;
        

/**************************************************************************
 **************************************************************************/
.ENDMOD;

